/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Encapsulates information on the current user.
 */
Ext.define( 'QDesk.CurrentUser', {

  requires : [
    'QDesk.Log',
    'QDesk.util.constant.EventName'
  ],

  singleton : true,

  /**
   * The current session token (HTTP session ID).
   */
  sessionToken : null,

  /**
   * Boolean flag indicating if the current user is authenticated.
   */
  authenticated : false,

  /**
   * User profile of the current user. Null if the current user is not authenticated.
   */
  userProfile : null,


  /**
   * Returns true if the current user is authenticated, false otherwise.
   * @returns {boolean} true if the current user is authenticated, false otherwise.
   */
  isAuthenticated : function ()
  {
    var me = this;
    return me.authenticated;
  },


  /**
   * Returns the user profile of the current user.
   * @returns the user profile of the current user.
   */
  getUserProfile : function ()
  {
    var me = this;
    return me.userProfile;
  },


  /**
   * Returns true if the current user has the specified permission, false otherwise.
   * @param permission a permission.
   * @return true if the current user has the specified permission, false otherwise.
   */
  hasPermission : function ( permission )
  {
    var me = this;
    return me.hasAnyPermissions( [ permission ] );
  },


  /**
   * Returns true if the current user has ANY of the specified permissions, false otherwise.
   * @param permissions permissions.
   * @return true if the current user has ANY of the specified permissions, false otherwise.
   */
  hasAnyPermissions : function ( permissions )
  {
    var me = this;

    var result = false;
    if ( me.isAuthenticated() )
    {
      if ( me.userProfile && me.userProfile.effectivePermission )
      {
        Ext.each( permissions, function ( permission )
        {
          if ( me.userProfile.effectivePermission.indexOf( permission ) !== -1 )
          {
            // effectivePermissions contains one of the requested permissions
            result = true;
            return false;  // stop iterations
          }
        } );
      }
    }

    return result;
  },


  /**
   * Checks if the current user has any of the specified permissions. If so, this method simply returns true.
   * Otherwise it shows a warning dialog with a warning message and returns false.

   * @param permissions permissions.
   * @return true if the current user has ANY of the specified permissions, false otherwise.
   */
  checkAnyPermissions : function ( permissions )
  {
    var me = this;

    var hasAnyPermissions = me.hasAnyPermissions( permissions );
    if ( !hasAnyPermissions )
    {
      var msg = QDesk.Msg.get( 'QDesk.userNotAuthorizedAnyPermissionsRequiredMsg',
          QDesk.util.FormatUtils.htmlFormatPermissions( permissions ) );

      QDesk.util.UxUtils.showWarningDialog( msg );
    }

    return hasAnyPermissions;
  },


  /**
   * Returns true if the current user has ALL of the specified permissions, false otherwise.
   * @param permissions permissions.
   * @return true if the current user has ALL of the specified permissions, false otherwise.
   */
  hasAllPermissions : function ( permissions )
  {
    var me = this;

    var result = false;
    if ( me.isAuthenticated() )
    {
      if ( me.userProfile && me.userProfile.effectivePermission )
      {
        result = true;  // assume the user has all permissions
        Ext.each( permissions, function ( permission )
        {
          if ( me.userProfile.effectivePermission.indexOf( permission ) === -1 )
          {
            // effectivePermissions does not contain one of the requested permissions
            result = false;
            return false;  // stop iterations
          }
        } );
      }
    }

    return result;
  },


  /**
   * Updates the current user's authentication status and user profile data using the provided user info object with the
   * following structure:
   * {
   *    authenticated : (boolean)
   *    userProfile : (UserProfile data present if the user is authenticated)
   * }
   *
   * @param sessionToken a session token.
   * @param userInfo user info data.
   * @param isFromHeartbeat a flag indicating if the update method was invoked from the heartbeat.
   */
  update : function ( sessionToken, userInfo, isFromHeartbeat )
  {
    var me = this;

    var oldAuthenticated = me.authenticated;
    var newAuthenticated = userInfo.authenticated;

    var oldEffectivePermissions = me.userProfile ? me.userProfile.effectivePermission : [];
    var newEffectivePermissions = userInfo.userProfile ? userInfo.userProfile.effectivePermission : [];

    me.sessionToken = sessionToken;
    me.authenticated = userInfo.authenticated;
    me.userProfile = userInfo.userProfile;

    if ( oldAuthenticated !== newAuthenticated )
    {
      QDesk.Log.info( Ext.String.format( "Current user's authentication status changed!\nOld: {0}\nNew: {1}",
          oldAuthenticated, newAuthenticated ) );

      QDesk.getApplication().fireEvent( QDesk.util.constant.EventName.CURRENT_USER_AUTH_STATUS_CHANGED, {
        sessionToken : sessionToken,
        oldStatus : oldAuthenticated,
        newStatus : newAuthenticated,
        isFromHeartbeat : isFromHeartbeat
      } );

      // AUTHENTICATED -> NOT-AUTHENTICATED and the update is from the heartbeat, then it means the user's session expired
      if ( oldAuthenticated && !newAuthenticated && isFromHeartbeat )
      {
        QDesk.getApplication().fireEvent( QDesk.util.constant.EventName.LOGOUT_SUCCESS, {
          sessionToken : sessionToken,
          automatic : true
        } );
      }
    }

    if ( me._permissionsChanged( oldEffectivePermissions, newEffectivePermissions ) )
    {
      QDesk.Log.info( Ext.String.format( "Current user's permissions changed!\nOld: [{0}]\nNew: [{1}]",
          oldEffectivePermissions, newEffectivePermissions ) );

      QDesk.getApplication().fireEvent( QDesk.util.constant.EventName.CURRENT_USER_PERMISSIONS_CHANGED, {
        sessionToken : sessionToken,
        oldEffectivePermissions : oldEffectivePermissions,
        newEffectivePermissions : newEffectivePermissions,
        isFromHeartbeat : isFromHeartbeat
      } );
    }
  },


  _permissionsChanged : function ( oldPermissions, newPermissions )
  {
    // shortcut
    if ( oldPermissions.length !== newPermissions.length )
      return true;

    // arrays have equal sizes - check if they contain identical elements
    var same = oldPermissions.every( function ( oldPermission )
    {
      return newPermissions.indexOf( oldPermission ) !== -1;
    } );

    return !same;
  }
} );
