/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * QuartzDesk JVM Agent info accessor.
 */
Ext.define( 'QDesk.JvmAgentInfoAccessor', {
  requires : [
    'QDesk.Log'
  ],

  /**
   * Required QuartzDesk JVM Agent Info config parameter with the following structure:
   * {
   *   detected : true/false,
   *   compatibleVersion : true/false,
   *   compatibleEdition : true/false,
   *   compatible : true/false,
   *   version : {
   *      major : 1,
   *      minor : 2,
   *      maintenance : 3,
   *      qualifier : foobar
   *   },
   *   edition : Lite/Standard/Enterprise,
   *   nameAndVersion : 'QuartzDesk JVM agent 1.2.3-foobar',
   *   displayVersion : '1.2.3-foobar',
   *   locale : 'US'
   *   timeZone : 'CET'
   *   availableTimeZones [
   *   ],
   *   configuration : [
   *     {
   *       name: config_param_name,
   *       value: config_param_value
   *     }
   *   ]
   *  }
   */
  data : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.data )
      Ext.Error.raise( 'Required config parameter missing: data' );

    me.data = config.data;

    me.callParent( [ config ] );
  },


  /**
   * Returns true, if the QuartzDesk JVM Agent is detected, false otherwise.
   * @return true if detected, false otherwise.
   */
  isDetected : function ()
  {
    var me = this;
    return me.data.detected;
  },


  /**
   * Returns true, if the QuartzDesk JVM Agent is ready (i.e. detected and compatible with the QuartzDesk GUI), false
   * otherwise.
   * @return true if ready, false otherwise.
   */
  isReady : function ()
  {
    var me = this;
    return me.data.ready;
  },


  /**
   * Returns true, if the QuartzDesk JVM Agent version is compatible with the installed QuartzDesk web application
   * version, false otherwise.
   * @return true if compatible, false otherwise.
   */
  isCompatibleVersion : function ()
  {
    var me = this;
    return me.data.compatibleVersion;
  },


  /**
   * Returns true, if the QuartzDesk JVM Agent edition is compatible with the installed QuartzDesk web application
   * edition, false otherwise.
   * @return true if compatible, false otherwise.
   */
  isCompatibleEdition : function ()
  {
    var me = this;
    return me.data.compatibleEdition;
  },


  /**
   * Returns true, if the QuartzDesk JVM Agent is compatible with the installed QuartzDesk web application, false
   * otherwise.
   * @return true if compatible, false otherwise.
   */
  isCompatible : function ()
  {
    var me = this;
    return me.data.compatible;
  },


  /**
   * Returns the object describing the version of the installed QuartzDesk JVM Agent.
   * @return the version object.
   */
  getVersion : function ()
  {
    var me = this;
    return me.data.version;
  },


  /**
   * Returns the QuartzDesk JVM Agent edition.
   * @return the edition.
   */
  getEdition : function ()
  {
    var me = this;
    return me.data.edition;
  },


  /**
   * Returns the QuartzDesk JVM Agent name and version.
   * @return the name and version.
   */
  getNameAndVersion : function ()
  {
    var me = this;
    return me.data.nameAndVersion;
  },


  /**
   * Returns the QuartzDesk JVM Agent display version.
   * @return the display version.
   */
  getDisplayVersion : function ()
  {
    var me = this;
    return me.data.displayVersion;
  },


  /**
   * Returns the QuartzDesk JVM Agent locale.
   * @return the locale.
   */
  getLocale : function ()
  {
    var me = this;
    return me.data.locale;
  },


  /**
   * Returns the QuartzDesk JVM Agent time zone.
   * @return the time zone.
   */
  getTimeZone : function ()
  {
    var me = this;
    return me.data.timeZone;
  },


  /**
   * Returns the QuartzDesk JVM Agent available time zones.
   * @return the available time zones.
   */
  getAvailableTimeZones : function ()
  {
    var me = this;
    return me.data.availableTimeZones;
  },


  /**
   * Returns the specified JVM Agent property value, null if not defined.
   * @param propertyKey a config property key.
   * @return the specified JVM Agent property value, or null.
   */
  getConfigProperty : function ( propertyKey )
  {
    var me = this;
    var configMap = new Ext.util.HashMap();
    Ext.each( me.data.configuration, function ( entry )
    {
      configMap.add( entry.name, entry.value );
    } );

    return configMap.get( propertyKey );
  },


  /**
   * Returns true, if the QuartzDesk JVM Agent is ready and the tracking of misfired triggers is enabled.
   * @return true if ready and tracking of misfired triggers is enabled, false otherwise.
   */
  isJvmAgentQuartzMisfiredTriggersTrackingEnabled : function ()
  {
    var me = this;
    if ( me.isReady() )
    {
      return me.getConfigProperty( 'quartz.misfiredTriggers.tracking.enabled' ) === 'true';
    }
    return false;
  },


  /**
   * Returns true, if the QuartzDesk JVM Agent is ready and the processing of misfired triggers is enabled.
   * @return true if ready and processing of misfired triggers is enabled, false otherwise.
   */
  isJvmAgentQuartzMisfiredTriggersProcessingEnabled : function ()
  {
    var me = this;
    if ( me.isReady() )
    {
      return me.getConfigProperty( 'quartz.misfiredTriggers.processing.enabled' ) === 'true';
    }
    return false;
  }
} );
