/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a window showing audit trail entries of the specified user.
 */
Ext.define( 'QDesk.controller.command.audit.ViewUserAuditTrail', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.constant.ActionUrl',
    'QDesk.view.work.audit.AuditTrailWindow'
  ],

  /**
   * Required config parameter with the ID of the user whose audit trail entries should be displayed.
   */
  userId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.userId )
      Ext.Error.raise( 'Required config parameter missing: userId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var window = Ext.widget( 'AuditTrailWindow', {
      qdesk : {
        gridId : 'userAuditTrailGrid'
      }
    } );

    me._bindEventHandlers( window );

    var auditTrailGrid = window.down( '#auditTrailGrid' );

    me._refreshAuditTrailGrid( auditTrailGrid );
  },


  _bindEventHandlers : function ( window )
  {
    var me = this;

    window.on( 'afterrender', function ()
    {
      window.maximize();
      window.toFront( false );
    } );

    var auditTrailGrid = window.down( '#auditTrailGrid' );

    // refresh button
    auditTrailGrid.on( 'refresh',
        function ()
        {
          me._refreshAuditTrailGrid( auditTrailGrid );
        } );

    // filters changed
    auditTrailGrid.on( 'changeFilters',
        function ()
        {
          me._refreshAuditTrailGrid( auditTrailGrid );
        } );

    // TODO: check audit trail index is enabled
    var auditTrailIndexEnabled = false;

    auditTrailGrid.on( 'loadFailed',
        function ( operation )
        {
          auditTrailGrid.down( '#refreshButton' ).enable();
        } );

    auditTrailGrid.on( 'loadSuccess',
        function ( store, records )
        {
          auditTrailGrid.down( '#refreshButton' ).enable();
          auditTrailGrid.down( '#fetchPeriod' ).enable();
          auditTrailGrid.down( '#maxFetchSizeSelect' ).enable();
          // auditTrailGrid.down( '#fetchRelatedCheckbox' ).enable();

          // enable only if the index is enabled
          if ( auditTrailIndexEnabled )
          {
            auditTrailGrid.down( '#filterQueryText' ).enable();
          }

          // update empty text if there are no records
          var view = auditTrailGrid.getView();
          if ( Ext.isEmpty( records ) )
          {
            view.emptyText =
                QDesk.Msg.get( 'QDesk.controller.command.audit.ViewUserAuditTrail.emptyTextNoMatchingData' );

            view.refresh();
          }

          // upon the initial load of the store, show the window
          if ( !window.rendered )
          {
            window.show();
          }
        } );
  },


  /**
   * Invoked to refresh the data in the audit trail grid.
   */
  _refreshAuditTrailGrid : function ( auditTrailGrid )
  {
    var me = this;

    var extraStoreParams = {
      userId : me.userId
    };

    var fetchPeriod = auditTrailGrid.down( '#fetchPeriod' ).getValue();
    Ext.apply( extraStoreParams, {
      fetchPeriod : Ext.encode( fetchPeriod )
    } );

    var maxFetchSize = auditTrailGrid.down( '#maxFetchSizeSelect' ).getValue();
    if ( maxFetchSize !== -1 )  // -1 == unlimited
    {
      Ext.apply( extraStoreParams, {
        maxFetchSize : maxFetchSize
      } );
    }

    // if audit trail index enabled, we add the filter query text param
    // TODO: check the index is enabled
    var auditTrailIndexEnabled = true;

    if ( auditTrailIndexEnabled )
    {
      var filterQueryText = auditTrailGrid.down( '#filterQueryText' ).getValue();
      if ( !Ext.isEmpty( filterQueryText ) )
      {
        Ext.apply( extraStoreParams, {
          filterQuery : filterQueryText
        } );
      }
    }

    var store = auditTrailGrid.getStore();
    store.getProxy().url = QDesk.util.constant.ActionUrl.auditTrailActionListForUser;
    store.getProxy().extraParams = extraStoreParams;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.audit.ViewUserAuditTrail.maskLoadMsg' ) );

    // initial load of data
    store.loadPage( 1, {
      callback : function ( records, operation, successful )
      {
        me.unmask();

        if ( successful )
        {
          // invoke the success callback handler
          if ( me.success && Ext.isFunction( me.success ) )
          {
            me.success.apply( me );
          }
        }
        else
        {
          var response = operation.getResponse();
          if ( response && response.responseText )
          {
            // application errors
            QDesk.util.UxUtils.showJsonResponseErrors( Ext.JSON.decode( response.responseText ),
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get( 'QDesk.controller.command.audit.ViewUserAuditTrail.errorMsg' ),
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
      }
    } );
  }
} );
