/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


/**
 * Scheduler connection related utility methods.
 */
Ext.define( 'QDesk.controller.command.connection.ConnectionUtils', {

  requires : [
    'QDesk.JvmAgentInfoAccessor',
    'QDesk.SchedulerInfoAccessor',
    'QDesk.model.scheduler.SchedulerConnection',
    'QDesk.util.EnumUtils',

    'QDesk.util.license.LicensedProduct'
  ],

  singleton : true,


  /**
   * Returns the item ID of the scheduler view panel for the specified connection.
   * @param connectionId a connection ID.
   */
  getSchedulerViewItemId : function ( connectionId )
  {
    return 'schedulerView-' + connectionId;
  },


  /**
   * Returns the scheduler view (panel) for the specified Quartz connection ID.
   * @param connectionId a connection ID.
   * @return the scheduler view.
   */
  getSchedulerView : function ( connectionId )
  {
    var viewItemId = QDesk.controller.command.connection.ConnectionUtils.getSchedulerViewItemId( connectionId );
    var views = Ext.ComponentQuery.query( '#mainTabPanel > #' + viewItemId );
    return Ext.isEmpty( views ) ? null : views[ 0 ];
  },


  /**
   * Returns true, if the QuartzDesk JVM Agent for the specified connection ID is ready (i.e. detected
   * and compatible with the QuartzDesk GUI).
   * @param connectionId a connection ID.
   * @return true if ready, false otherwise.
   */
  isJvmAgentReady : function ( connectionId )
  {
    var jvmAgentInfo = QDesk.controller.command.connection.ConnectionUtils.getJvmAgentInfo( connectionId );
    return jvmAgentInfo.isReady();
  },


  /**
   * Returns the QuartzDesk JVM Agent API warning message to show in the GUI if a QuartzDesk JVM Agent
   * installation-related problem has been detected.
   * @param connectionId a connection ID.
   * @returns the warning message.
   */
  getJvmAgentWarningMessage : function ( connectionId )
  {
    var schedulerInfo = QDesk.controller.command.connection.ConnectionUtils.getSchedulerInfo( connectionId );
    var jvmAgentInfo = QDesk.controller.command.connection.ConnectionUtils.getJvmAgentInfo( connectionId );

    if ( jvmAgentInfo.isReady() )
      return null;

    if ( jvmAgentInfo.isDetected() )
    {
      if ( !jvmAgentInfo.isCompatibleVersion() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentIncompatibleVersionMsg',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            QDesk.util.FormatUtils.formatProductVersion( jvmAgentInfo.getVersion() ),
            QDesk.util.FormatUtils.formatProductVersion( QDesk.util.license.LicensedProduct.productVersion ) );
      }
      else if ( !jvmAgentInfo.isCompatibleEdition() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentIncompatibleEditionMsg',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            jvmAgentInfo.getEdition(),
            QDesk.util.license.LicensedProduct.productEdition );
      }
      else
      {
        return null;
      }
    }
    else
    {
      return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentNotDetectedMsg',
          QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
    }
  },


  /**
   * Returns the QuartzDesk JVM Agent API warning tooltip to show in the GUI if a QuartzDesk JVM Agent
   * installation-related problem has been detected.
   * @param connectionId a connection ID.
   * @returns the warning message.
   */
  getJvmAgentWarningTooltip : function ( connectionId )
  {
    var schedulerInfo = QDesk.controller.command.connection.ConnectionUtils.getSchedulerInfo( connectionId );
    var jvmAgentInfo = QDesk.controller.command.connection.ConnectionUtils.getJvmAgentInfo( connectionId );

    if ( jvmAgentInfo.isReady() )
      return null;

    if ( jvmAgentInfo.isDetected() )
    {
      if ( !jvmAgentInfo.isCompatibleVersion() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentIncompatibleVersionTooltip',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            QDesk.util.FormatUtils.formatProductVersion( jvmAgentInfo.getVersion() ),
            QDesk.util.FormatUtils.formatProductVersion( QDesk.util.license.LicensedProduct.productVersion ) );
      }
      else if ( !jvmAgentInfo.isCompatibleEdition() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentIncompatibleEditionTooltip',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            jvmAgentInfo.getEdition(),
            QDesk.util.license.LicensedProduct.productEdition );
      }
      else
      {
        return null;
      }
    }
    else
    {
      return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentNotDetectedTooltip',
          QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
    }
  },


  /**
   * Returns the QuartzDesk Public API warning message to show in the GUI if a QuartzDesk Public API
   * installation-related problem has been detected.
   * @param connectionId a connection ID.
   * @returns the warning message.
   */
  getPublicApiWarningMessage : function ( connectionId )
  {
    var schedulerInfo = QDesk.controller.command.connection.ConnectionUtils.getSchedulerInfo( connectionId );

    if ( schedulerInfo.isPublicApiReady() )
      return null;

    if ( schedulerInfo.isPublicApiDetected() )
    {
      if ( !schedulerInfo.isPublicApiLoadedCorrectly() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiNotLoadedCorrectlyMsg',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
      }
      else if ( !schedulerInfo.isPublicApiCompatibleVersion() )
      {
        var jvmAgentInfo = QDesk.controller.command.connection.ConnectionUtils.getJvmAgentInfo( connectionId );

        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiIncompatibleVersionMsg',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            QDesk.util.FormatUtils.formatProductVersion( schedulerInfo.getPublicApiVersion() ),
            QDesk.util.FormatUtils.formatProductVersion( jvmAgentInfo.getVersion() ) );
      }
      else
      {
        return null;
      }
    }
    else
    {
      return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiNotDetectedMsg',
          QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
    }
  },


  /**
   * Returns the QuartzDesk Public API warning tooltip to show in the GUI if a QuartzDesk Public API
   * installation-related problem has been detected.
   * @param connectionId a connection ID.
   * @returns the warning message.
   */
  getPublicApiWarningTooltip : function ( connectionId )
  {
    var schedulerInfo = QDesk.controller.command.connection.ConnectionUtils.getSchedulerInfo( connectionId );

    if ( schedulerInfo.isPublicApiReady() )
      return null;

    if ( schedulerInfo.isPublicApiDetected() )
    {
      if ( !schedulerInfo.isPublicApiLoadedCorrectly() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiNotLoadedCorrectlyTooltip',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
      }
      else if ( !schedulerInfo.isPublicApiCompatibleVersion() )
      {
        var jvmAgentInfo = QDesk.controller.command.connection.ConnectionUtils.getJvmAgentInfo( connectionId );

        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiIncompatibleVersionTooltip',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            QDesk.util.FormatUtils.formatProductVersion( schedulerInfo.getPublicApiVersion() ),
            QDesk.util.FormatUtils.formatProductVersion( jvmAgentInfo.getVersion() ) );
      }
      else
      {
        return null;
      }
    }
    else
    {
      return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiNotDetectedTooltip',
          QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
    }
  },


  /**
   * Returns the "The feature is not available in ... edition.' warning message.
   * @returns the warning message.
   */
  getNotLicensedWarningMessage : function ()
  {
    return QDesk.Msg.get( 'QDesk.featureAccessRestricted.notLicensedMsg',
        QDesk.util.license.LicensedProduct.productName )
  },


  /**
   * Returns the "The feature is not available in ... edition.' warning tooltip.
   * @returns the warning message.
   */
  getNotLicensedWarningTooltip : function ()
  {
    return QDesk.Msg.get( 'QDesk.featureAccessRestricted.notLicensedTooltip',
        QDesk.util.license.LicensedProduct.productName )
  },


  /**
   * Returns true, if the QuartzDesk JVM Agent for the specified connection ID is available
   * and compatible and the jobs index is enabled in the agent.
   * @param connectionId a connection ID.
   * @return true if available, compatible and jobs index enabled, false otherwise.
   */
  isJvmAgentQuartzJobsIndexEnabled : function ( connectionId )
  {
    var me = this;
    if ( me.isJvmAgentReady( connectionId ) )
    {
      return QDesk.controller.command.connection.ConnectionUtils.getJvmAgentConfigProperty(
          connectionId, 'quartz.index.jobs.enabled' ) == 'true';
    }
    return false;
  },


  /**
   * Returns true, if the QuartzDesk JVM Agent for the specified connection ID is available
   * and compatible and the triggers index is enabled in the agent.
   * @param connectionId a connection ID.
   * @return true if available, compatible and triggers index enabled, false otherwise.
   */
  isJvmAgentQuartzTriggersIndexEnabled : function ( connectionId )
  {
    var me = this;
    if ( me.isJvmAgentReady( connectionId ) )
    {
      return QDesk.controller.command.connection.ConnectionUtils.getJvmAgentConfigProperty(
          connectionId, 'quartz.index.triggers.enabled' ) == 'true';
    }
    return false;
  },


  /**
   * Returns true, if the QuartzDesk JVM Agent for the specified connection ID is available
   * and compatible and the misfired triggers index is enabled in the agent.
   * @param connectionId a connection ID.
   * @return true if available, compatible and misfired trigger index enabled, false otherwise.
   */
  isJvmAgentQuartzMisfiredTriggersIndexEnabled : function ( connectionId )
  {
    var me = this;

    if ( me.isJvmAgentReady( connectionId ) )
    {
      return QDesk.controller.command.connection.ConnectionUtils.getJvmAgentConfigProperty(
          connectionId, 'quartz.index.misfiredTriggers.enabled' ) == 'true';
    }
    return false;
  },


  /**
   * Returns true, if the QuartzDesk JVM Agent for the specified connection ID is available
   * and compatible and the exec history index is enabled in the agent.
   * @param connectionId a connection ID.
   * @return true if available, compatible and exec history index enabled, false otherwise.
   */
  isJvmAgentQuartzExecHistoryIndexEnabled : function ( connectionId )
  {
    var me = this;
    if ( me.isJvmAgentReady( connectionId ) )
    {
      return QDesk.controller.command.connection.ConnectionUtils.getJvmAgentConfigProperty(
          connectionId, 'quartz.index.execHistory.enabled' ) == 'true';
    }
    return false;
  },


  /**
   * Returns true, if the QuartzDesk JVM Agent for the specified connection ID is available
   * and compatible and the notification rules index is enabled in the agent.
   * @param connectionId a connection ID.
   * @return true if available, compatible and notification rules index enabled, false otherwise.
   */
  isJvmAgentQuartzExecNotificationRulesIndexEnabled : function ( connectionId )
  {
    var me = this;
    if ( me.isJvmAgentReady( connectionId ) )
    {
      return QDesk.controller.command.connection.ConnectionUtils.getJvmAgentConfigProperty(
          connectionId, 'quartz.index.execNotificationRules.enabled' ) == 'true';
    }
    return false;
  },


  /**
   * Returns true, if the QuartzDesk JVM Agent for the specified connection ID is available
   * and compatible and the job chains index is enabled in the agent.
   * @param connectionId a connection ID.
   * @return true if available, compatible and job chains index enabled, false otherwise.
   */
  isJvmAgentQuartzJobChainsIndexEnabled : function ( connectionId )
  {
    var me = this;
    if ( me.isJvmAgentReady( connectionId ) )
    {
      return QDesk.controller.command.connection.ConnectionUtils.getJvmAgentConfigProperty(
          connectionId, 'quartz.index.jobChains.enabled' ) == 'true';
    }
    return false;
  },


  /**
   * Returns the specified JVM Agent property value, null if not defined.
   * @param connectionId a connection ID.
   * @param propertyKey a config property key.
   * @return the specified JVM Agent property value, or null.
   */
  getJvmAgentConfigProperty : function ( connectionId, propertyKey )
  {
    var jvmAgentInfo = QDesk.controller.command.connection.ConnectionUtils.getJvmAgentInfo( connectionId );
    return jvmAgentInfo.getConfigProperty( propertyKey );
  },


  /**
   * Returns the QDesk.JvmAgentInfo object for the specified connection ID.
   * @param connectionId a connection ID.
   * @return the QDesk.JvmAgentInfo object.
   */
  getJvmAgentInfo : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    var jvmAgentInfo = schedulerView.getJvmAgentInfo();
    if ( !jvmAgentInfo )
      Ext.Error.raise( 'QuartzDesk JVM Agent info for connection ID: ' + connectionId + ' not found.' );

    return jvmAgentInfo;
  },


  /**
   * Returns true if the QuartzDesk Public API Library has been detected on the scheduler's classpath, is correctly
   * installed and its version is compatible with the agent.
   * @param connectionId a connection ID.
   * @returns true if ready, false otherwise.
   */
  isPublicApiReady : function ( connectionId )
  {
    var schedulerInfo = QDesk.controller.command.connection.ConnectionUtils.getSchedulerInfo( connectionId );
    return schedulerInfo.isPublicApiReady();
  },


  /**
   * Returns the QuartzDesk Public API Info object for the specified connection ID.
   * @param connectionId a connection ID.
   * @return the info object with the following structure:
   * <pre>
   * {
   *   "detected" : true,
   *   "ready" : true,
   *   "loadedCorrectly" : true
   *   "compatibleVersion: true
   *   "version" : {
   *     "major" : 1,
   *     "minor" : 3,
   *     "maintenance" : 0,
   *     "qualifier" : "SNAPSHOT"
   *   }
   * }
   * </pre>
   */
  getPublicApiInfo : function ( connectionId )
  {
    var schedulerInfo = QDesk.controller.command.connection.ConnectionUtils.getSchedulerInfo( connectionId );

    var publicApiInfo = schedulerInfo.publicApiInfo;
    if ( !publicApiInfo )
      Ext.Error.raise( 'QuartzDesk Public API info for connection ID: ' + connectionId + ' not found.' );

    return publicApiInfo;
  },


  /**
   * Returns the QDesk.SchedulerInfoAccessor object for the specified connection ID.
   * @param connectionId a connection ID.
   * @return the QDesk.SchedulerInfoAccessor object.
   */
  getSchedulerInfo : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    var schedulerInfo = schedulerView.getSchedulerInfo();
    if ( !schedulerInfo )
      Ext.Error.raise( 'Scheduler info for connection ID: ' + connectionId + ' not found.' );

    return schedulerInfo;
  },


  /**
   * Returns the node ID of the specified connection.
   * @param connectionId a connection ID.
   * @return the node ID of the specified connection.
   */
  getConnectionNodeId : function ( connectionId )
  {
    return 'connection-' + connectionId;
  },


  /**
   * Returns the name of the connection with the specified ID.
   * @param connectionId a connection ID.
   */
  getConnectionName : function ( connectionId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[ 0 ];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getConnectionNodeId( connectionId );
    var connectionNode = store.getNodeById( nodeId );
    return connectionNode.get( 'name' );
  },


  /**
   * Returns the title of the connection panel for the connection with the specified ID.
   * @param connectionId a connection ID.
   */
  getConnectionPanelTitle : function ( connectionId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[ 0 ];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getConnectionNodeId( connectionId );
    var connectionNode = store.getNodeById( nodeId );
    return connectionNode.get( 'name' );
  },


  /**
   * Returns the ID of the parent folder of the specified connection, or null if not found.
   * @param connectionId a connection ID.
   * @return the parent folder ID, or null.
   */
  getConnectionParentFolderId : function ( connectionId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[ 0 ];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getConnectionNodeId( connectionId );
    var connectionNode = store.getNodeById( nodeId );
    return connectionNode && connectionNode.parentNode ? connectionNode.parentNode.get( 'folderId' ) : null;
  },


  /**
   * Returns the node ID of the specified folder.
   * @param folderId a folder ID.
   * @return the node ID of the specified folder.
   */
  getFolderNodeId : function ( folderId )
  {
    return folderId == QDesk.util.constant.Const.CONNECTION_TREE_ROOT_FOLDER_ID ?
        QDesk.util.constant.Const.CONNECTION_TREE_ROOT_ID : 'folder-' + folderId;
  },


  /**
   * Returns the name of the folder with the specified ID.
   * @param folderId a folder ID.
   * @return the folder name.
   */
  getFolderName : function ( folderId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[ 0 ];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getFolderNodeId( folderId );
    var folderNode = store.getNodeById( nodeId );
    return folderNode.get( 'name' );
  },


  /**
   * Returns the path of the folder with the specified ID.
   * @param folderId a folder ID.
   * @return the folder path.
   */
  getFolderPath : function ( folderId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[ 0 ];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getFolderNodeId( folderId );
    var folderNode = store.getNodeById( nodeId );
    return folderNode.getPath();
  },


  /**
   * Returns the ID of the parent folder of the specified folder, or null if not found.
   * @param folderId a folder ID.
   * @return the parent folder ID, or null.
   */
  getFolderParentFolderId : function ( folderId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[ 0 ];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getFolderNodeId( folderId );
    var folderNode = store.getNodeById( nodeId );
    return folderNode && folderNode.parentNode ? folderNode.parentNode.get( 'folderId' ) : null;
  },


  /**
   * Returns the scheduler type for the specified connection ID.
   * @param connectionId a connection ID.
   * @return the scheduler type (QDesk.model.scheduler.SchedulerType value).
   */
  getSchedulerType : function ( connectionId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[ 0 ];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getConnectionNodeId( connectionId );
    var connectionNode = store.getNodeById( nodeId );
    return connectionNode.get( 'schedulerType' );
  },


  /**
   * Creates a new {@link QDesk.model.scheduler.SchedulerConnection} instance and populates it
   * from the specified EditSchedulerConnectionWindow window.
   * @param schedulerType a scheduler type ({@link QDesk.model.scheduler.SchedulerType} enum value).
   * @param window an EditSchedulerConnectionWindow window.
   * @returns the created {QDesk.model.scheduler.SchedulerConnection} instance.
   */
  createSchedulerConnection : function ( schedulerType, window )
  {
    var me = this;

    var connectionDataFormPanel = window.down( '#connectionDataForm' );

    var useServiceParams = connectionDataFormPanel.down( '#serviceParamsRadio' ).getValue();
    var useServiceUrl = connectionDataFormPanel.down( '#serviceUrlRadio' ).getValue();

    var form = connectionDataFormPanel.getForm();
    var fieldValues = form.getFieldValues();

    var connection = Ext.create( 'QDesk.model.scheduler.SchedulerConnection',
        {
          id : fieldValues.id,
          //createdAt : fieldValues.createdAt,
          //updatedAt : fieldValues.updatedAt,
          name : fieldValues.name,
          remoteJmxService : {
            protocol : useServiceParams ? fieldValues.protocol : null,
            host : useServiceParams ? fieldValues.host : null,
            port : useServiceParams ? fieldValues.port : null,
            serviceUrl : useServiceUrl ? fieldValues.serviceUrl : null,
            authentication : {
              username : fieldValues.username ? fieldValues.username : null,
              password : fieldValues.password ? fieldValues.password : null
            },
            secure : fieldValues.secure == null ? false : fieldValues.secure
          },
          schedulerType : schedulerType,
          schedulerObjectName : fieldValues.schedulerObjectName
        }
    );

    if ( fieldValues.username == '' && fieldValues.password == '' )
      connection.get( 'remoteJmxService' ).authentication = null;

    /*
     * Collect config entry values.
     */
    var configEntries = [];
    Ext.each( Ext.ComponentQuery.query( 'form', window ), function ( formPanel )
    {
      // skip connectionDataForm - config values are on other forms
      if ( formPanel.getItemId() != 'connectionDataForm' )
      {
        var fieldValues = formPanel.getForm().getFieldValues();
        Ext.iterate( fieldValues, function ( name )
        {
          configEntries.push( {
            name : name,
            value : fieldValues[ name ]
          } );
        } );
      }
    } );
    connection.set( 'configEntry', configEntries );

    return connection;
  },


  handleTestResponse : function ( json, schedulerType )
  {
    if ( json.scheduler )
    {
      // Lite edition
      if ( QDesk.util.license.LicensedProduct.productEdition == QDesk.model.license.ProductEdition.LITE )
      {
        QDesk.util.UxUtils.showInfoDialog(
            QDesk.Msg.get(
                'QDesk.controller.command.connection.ConnectionUtils.connectionSuccessMsg' ) );
      }
      else
      {
        // Standard and Enterprise editions
        if ( json.jvmAgentReady )
        {
          //
          // jvm agent is ready
          //
          var publicApiInfo = json.scheduler.publicApiInfo;

          if ( publicApiInfo.ready )
          {
            // public API is ready
            QDesk.util.UxUtils.showInfoDialog(
                QDesk.Msg.get(
                    'QDesk.controller.command.connection.ConnectionUtils.connectionSuccessJvmAgentReadyPublicApiReadyMsg',
                    QDesk.util.EnumUtils.getSchedulerTypeName( schedulerType ) ) );
          }
          else
          {
            // public API is not ready
            if ( publicApiInfo.detected )
            {
              if ( !publicApiInfo.loadedCorrectly )
              {
                QDesk.util.UxUtils.showWarningDialog(
                    QDesk.Msg.get(
                        'QDesk.controller.command.connection.ConnectionUtils.connectionSuccessJvmAgentReadyPublicApiNotLoadedCorrectlyMsg',
                        QDesk.util.EnumUtils.getSchedulerTypeName( schedulerType ) ) );
              }
              else if ( !publicApiInfo.compatibleVersion )
              {
                QDesk.util.UxUtils.showWarningDialog(
                    QDesk.Msg.get(
                        'QDesk.controller.command.connection.ConnectionUtils.connectionSuccessJvmAgentReadyPublicApiIncompatibleVersionMsg',
                        QDesk.util.EnumUtils.getSchedulerTypeName( schedulerType ),
                        QDesk.util.FormatUtils.formatProductVersion( publicApiInfo.version ),
                        QDesk.util.FormatUtils.formatProductVersion( json.jvmAgentVersion ) ) );
              }
            }
            else
            {
              QDesk.util.UxUtils.showWarningDialog(
                  QDesk.Msg.get(
                      'QDesk.controller.command.connection.ConnectionUtils.connectionSuccessJvmAgentReadyPublicApiNotDetectedMsg',
                      QDesk.util.EnumUtils.getSchedulerTypeName( schedulerType ) ) );
            }
          }
        }
        else
        {
          //
          // jvm agent is not ready
          //
          if ( json.jvmAgentDetected )
          {
            // jvm agent detected

            if ( !json.jvmAgentCompatibleVersion )
            {
              QDesk.util.UxUtils.showWarningDialog(
                  QDesk.Msg.get(
                      'QDesk.controller.command.connection.ConnectionUtils.connectionSuccessJvmAgentIncompatibleVersionMsg',
                      QDesk.util.EnumUtils.getSchedulerTypeName( schedulerType ),
                      QDesk.util.FormatUtils.formatProductVersion( json.jvmAgentVersion ),
                      QDesk.util.FormatUtils.formatProductVersion(
                          QDesk.util.license.LicensedProduct.productVersion ) ) );
            }
            else if ( !json.jvmAgentCompatibleEdition )
            {
              QDesk.util.UxUtils.showWarningDialog(
                  QDesk.Msg.get(
                      'QDesk.controller.command.connection.ConnectionUtils.connectionSuccessJvmAgentIncompatibleEditionMsg',
                      QDesk.util.EnumUtils.getSchedulerTypeName( schedulerType ),
                      json.jvmAgentEdition,
                      QDesk.util.license.LicensedProduct.productEdition ) );
            }
          }
          else
          {
            //
            // jvm agent not detected
            //
            QDesk.util.UxUtils.showWarningDialog(
                QDesk.Msg.get(
                    'QDesk.controller.command.connection.ConnectionUtils.connectionSuccessJvmAgentNotDetectedMsg',
                    QDesk.util.EnumUtils.getSchedulerTypeName( schedulerType ) ) );
          }
        }
      }
    }
    else
    {
      QDesk.util.UxUtils.showWarningDialog(
          QDesk.Msg.get(
              'QDesk.controller.command.connection.ConnectionUtils.noSchedulerFoundMsg' ) );
    }
  }
} );
