/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Adds new, or updates an existing scheduler connection.
 */
Ext.define( 'QDesk.controller.command.connection.SaveSchedulerConnection', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.controller.command.connection.RefreshConnectionFolder',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.EventName',
    'QDesk.model.gui.ServerErrorType'
  ],

  /**
   * Required config parameter when saving new scheduler connection.
   */
  parentFolderId : undefined,

  /**
   * Required config parameter with the connection to add/update.
   */
  connection : undefined,

  /**
   * Required config parameter with the EditSchedulerConnectionWindow.
   */
  window : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connection )
      Ext.Error.raise( 'Required config parameter missing: connection' );

    if ( !config.window )
      Ext.Error.raise( 'Required config parameter missing: window' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    if ( Ext.isNumber( me.connection.get( 'id' ) ) )
    {
      me._updateConnection();
    }
    else
    {
      me._addConnection();
    }
  },


  _addConnection : function ()
  {
    var me = this;

    var connectionName = me.connection.get( 'name' );

    /*
     * Important to drop id property from the data to avoid JSOn deserialization issues on the server-side.
     */
    var connectionData = me.connection.getData();
    delete connectionData.id;

    QDesk.util.UxUtils.showWaitDialog(
        QDesk.Msg.get( 'QDesk.controller.command.connection.SaveSchedulerConnection.addWaitMsg',
            Ext.String.htmlEncode( connectionName ) ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.connectionManagementActionAddConnection,
      params : {
        parentFolderId : me.parentFolderId,
        connection : Ext.encode( connectionData )
      },
      callback : function ( opts, success, resp )
      {
        // hide the wait dialog
        Ext.MessageBox.hide();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            me.window.close();

            me.application.fireEvent( QDesk.util.constant.EventName.CONNECTION_ADDED, {
              connectionId : json.data.id
            } );

            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.connection.SaveSchedulerConnection.addSuccessMsg',
                    Ext.String.htmlEncode( connectionName ) ),
                function ()
                {
                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );

            /*
             * Refresh connection's parent folder in the connections tree view.
             */
            Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
              folderId : me.parentFolderId
            } ).execute();
          }
          else
          {
            if ( json.errorType && json.errorType == QDesk.model.gui.ServerErrorType.VALIDATION )
            {
              Ext.each( Ext.ComponentQuery.query( 'form', me.window ), function ( formPanel )
              {
                formPanel.getForm().markInvalid( json.errors );
              } );
            }

            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.connection.SaveSchedulerConnection.addErrorMsg',
                  Ext.String.htmlEncode( connectionName ) ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  },


  _updateConnection : function ()
  {
    var me = this;

    var connectionId = me.connection.get( 'id' );
    var connectionName = me.connection.get( 'name' );

    QDesk.util.UxUtils.showWaitDialog(
        QDesk.Msg.get( 'QDesk.controller.command.connection.SaveSchedulerConnection.updateWaitMsg',
            Ext.String.htmlEncode( connectionName ) ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.connectionManagementActionUpdateConnection,
      params : {
        connectionId : connectionId,
        connection : Ext.encode( me.connection.getData() )
      },
      callback : function ( opts, success, resp )
      {
        // hide the wait dialog
        Ext.MessageBox.hide();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            me.window.close();

            me.application.fireEvent( QDesk.util.constant.EventName.CONNECTION_UPDATED, {
              connectionId : connectionId
            } );

            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.connection.SaveSchedulerConnection.updateSuccessMsg',
                    Ext.String.htmlEncode( connectionName ) ),
                function ()
                {
                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );

            /*
             * Refresh connection's parent folder in the connections tree view.
             */
            var parentFolderId = QDesk.controller.command.connection.ConnectionUtils.getConnectionParentFolderId(
                connectionId );

            if ( parentFolderId )
            {
              Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
                folderId : parentFolderId
              } ).execute();
            }
            else
            {
              Ext.Error.raise( 'Parent folder ID for connection ID: ' + connectionId + ' not found.' );
            }
          }
          else
          {
            if ( json.errorType && json.errorType == QDesk.model.gui.ServerErrorType.VALIDATION )
            {
              Ext.each( Ext.ComponentQuery.query( 'form', me.window ), function ( formPanel )
              {
                formPanel.getForm().markInvalid( json.errors );
              } );
            }

            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.connection.SaveSchedulerConnection.updateErrorMsg',
                  Ext.String.htmlEncode( connectionName ) ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  }
} );
