/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens the message channel profiles window that allows one to add/edit and delete
 * message channel profiles.
 */
Ext.define( 'QDesk.controller.command.message.channel.EditMessageChannelProfiles', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.model.security.Permission',
    'QDesk.controller.command.message.channel.AddMessageChannelProfile',
    'QDesk.controller.command.message.channel.AddMessageChannelProfileFolder',
    'QDesk.controller.command.message.channel.DeleteMessageChannelProfile',
    'QDesk.controller.command.message.channel.DeleteMessageChannelProfileFolder',
    'QDesk.controller.command.message.channel.EditMessageChannelProfile',
    'QDesk.controller.command.message.channel.EditMessageChannelProfileFolder',
    'QDesk.controller.command.message.channel.MoveMessageChannelProfile',
    'QDesk.controller.command.message.channel.MoveMessageChannelProfileFolder',
    'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder',
    'QDesk.util.MessageChannelProfileUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.Const',
    'QDesk.view.work.message.channel.MessageChannelProfilesWindow'
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var window = Ext.widget( 'MessageChannelProfilesWindow', {
      qdesk : {}
    } );

    me._bindEventHandlers( window );

    window.show();

    window.toFront();
  },


  /**
   * Bind message channel profiles window event handlers.
   * @param window a message channel profiles window.
   */
  _bindEventHandlers : function ( window )
  {
    var me = this;

    // tree reload events
    var treePanel = window.down( '#messageChannelProfilesTreePanel' );
    var toolBar = treePanel.down( '#toolBar' );
    var treeStore = treePanel.getStore();

    // disable tree toolbar buttons before tree load
    treeStore.on( 'beforeload',
        function ( store, operation )
        {
          Ext.each( Ext.ComponentQuery.query( 'button', toolBar ), function ( button )
          {
            button.disable();
          } );
        }, me );

    treeStore.on( 'load',
        function ( store, records, success, operation )
        {
          var me = this;
          toolBar.down( '#refreshButton' ).enable();

          if ( success )
          {
            toolBar.down( '#collapseAllButton' ).enable();
          }
          else
          {
            var response = operation.getResponse();
            if ( response && response.responseText )
            {
              // application errors
              QDesk.util.UxUtils.showJsonResponseErrors( Ext.JSON.decode( response.responseText ) );
            }
            else
            {
              QDesk.util.UxUtils.showErrorDialog(
                  QDesk.Msg.get(
                      'QDesk.controller.command.message.channel.EditMessageChannelProfiles.errorLoadingProfilesMsg' ) );
            }
          }
        }, me );

    // context menu
    treePanel.on( 'itemcontextmenu',
        function ( view, record, item, index, event )
        {
          var nodeId = record.get( 'id' );
          event.stopEvent();

          var nodeType = record.get( 'nodeType' );
          if ( nodeType == 'folder' )
          {
            var folderCtxMenu = me._createFolderCtxMenu( record );
            folderCtxMenu.showAt( event.getXY() );
          }
          else if ( nodeType == 'profile' )
          {
            var profileCtxMenu = me._createProfileCtxMenu( record );
            profileCtxMenu.showAt( event.getXY() );
          }
        }, me );

    // selection changes
    treePanel.on( 'selectionchange',
        function ( selectionModel, selectedRecords )
        {
          if ( selectedRecords.length > 0 )  // selection change event is also fired after deselection
          {
            var selectedRecord = selectedRecords[ 0 ];
            var nodeId = selectedRecord.get( 'id' );
            var nodeType = selectedRecord.get( 'nodeType' );
            if ( nodeType == 'folder' )
            {
              me._onFolderSelected( selectedRecord );
            }
            else if ( nodeType == 'profile' )
            {
              me._onProfileSelected( selectedRecord );
            }
          }
          else
          {
            me._onUnselected();
          }
        }, me );

    var treeView = treePanel.getView();

    treeView.on( 'beforedrop', function ( node, data, overModel, dropPosition, dropFunction, eOpts )
    {
      var draggedRecord = data.records[ 0 ];
      var targetRecord = overModel;

      var draggedRecordParent = draggedRecord.parentNode;

      // if node is dragged to its existing parent, then we cancel the operation
      if ( draggedRecordParent.get( 'id' ) == targetRecord.get( 'id' ) )
        return false;

      if ( !me._checkNodeMove( draggedRecord, targetRecord ) )
      {
        return false;
      }

      me._onNodeDrag( draggedRecord, targetRecord );

      return 1;
    } );

// Commented out - target folder refresh is done in the MoveMessageChannelProfile
// and MoveMessageChannelProfileFolder commands.
//
//    treeView.on( 'drop', function ( node, data, overModel, dropPosition, eOpts )
//    {
//      var draggedRecord = data.records[0];
//      var targetRecord = overModel;
//      var targetFolderId = targetRecord.get( 'folderId' );
//      Ext.create( 'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder', {
//        folderId: targetFolderId
//      } ).execute();
//    } );

    me._bindToolbarEventHandlers( window );
  },


  /**
   * Binds event handlers to message channel profiles tree panel's tool bar.
   * @param window a message channel profiles window.
   */
  _bindToolbarEventHandlers : function ( window )
  {
    var me = this;

    // tree reload events
    var treePanel = window.down( '#messageChannelProfilesTreePanel' );
    var toolBar = treePanel.down( '#toolBar' );

    toolBar.down( '#refreshButton' ).on( 'click', me._onRefreshButtonClick, me );
    toolBar.down( '#collapseAllButton' ).on( 'click', me._onCollapseAllButtonClick, me );
    toolBar.down( '#deleteFolderOrProfileButton' ).on( 'click', me._onDeleteFolderOrProfileButtonClick, me );

    // add folder
    toolBar.down( '#addFolderButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[ 0 ].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfileFolder', {
          maskEl : window.getEl(),
          parentFolderId : folderId
        } ).execute();
      }
    }, me );

    /* Add message channel profile */

    // COLLABORATION_HIPCHAT
    toolBar.down( '#addCollaborationHipChatProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[ 0 ].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.model.message.channel.MessageChannel.COLLABORATION_HIPCHAT
        } ).execute();
      }
    }, me );

    // COLLABORATION_SLACK
    toolBar.down( '#addCollaborationSlackProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[ 0 ].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.model.message.channel.MessageChannel.COLLABORATION_SLACK
        } ).execute();
      }
    }, me );


    // EMAIL
    toolBar.down( '#addEmailProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[ 0 ].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.model.message.channel.MessageChannel.EMAIL
        } ).execute();
      }
    }, me );


    // IM_AIM
    toolBar.down( '#addImAimProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[ 0 ].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.model.message.channel.MessageChannel.IM_AIM
        } ).execute();
      }
    }, me );

    // IM_GTALK
    toolBar.down( '#addImGTalkProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[ 0 ].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.model.message.channel.MessageChannel.IM_GTALK
        } ).execute();
      }
    }, me );

    // IM_ICQ
    toolBar.down( '#addImIcqProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[ 0 ].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.model.message.channel.MessageChannel.IM_ICQ
        } ).execute();
      }
    }, me );

    // IM_XMPP
    toolBar.down( '#addImXmppProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[ 0 ].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.model.message.channel.MessageChannel.IM_XMPP
        } ).execute();
      }
    }, me );

    // IM_YAHOO
    toolBar.down( '#addImYahooProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[ 0 ].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.model.message.channel.MessageChannel.IM_YAHOO
        } ).execute();
      }
    }, me );


    // SMS_CLICKATELL_CENTRAL
    toolBar.down( '#addSmsClickatellCentralProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[ 0 ].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.model.message.channel.MessageChannel.SMS_CLICKATELL_CENTRAL
        } ).execute();
      }
    }, me );

    // SMS_CLICKATELL_PLATFORM
    toolBar.down( '#addSmsClickatellPlatformProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[ 0 ].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.model.message.channel.MessageChannel.SMS_CLICKATELL_PLATFORM
        } ).execute();
      }
    }, me );

    // SMS_CLOCKWORK
    toolBar.down( '#addSmsClockworkProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[ 0 ].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.model.message.channel.MessageChannel.SMS_CLOCKWORK
        } ).execute();
      }
    }, me );

    // SMS_NEXMO
    toolBar.down( '#addSmsNexmoProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[ 0 ].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.model.message.channel.MessageChannel.SMS_NEXMO
        } ).execute();
      }
    }, me );

    // SMS_SMSGLOBAL
    toolBar.down( '#addSmsSmsGlobalProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[ 0 ].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.model.message.channel.MessageChannel.SMS_SMSGLOBAL
        } ).execute();
      }
    }, me );

    // SMS_TWILIO
    toolBar.down( '#addSmsTwilioProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[ 0 ].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.model.message.channel.MessageChannel.SMS_TWILIO
        } ).execute();
      }
    }, me );


    // SNMPV1_TRAP
    toolBar.down( '#addSnmpV1TrapProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[ 0 ].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.model.message.channel.MessageChannel.SNMPV1_TRAP
        } ).execute();
      }
    }, me );

    // SNMPV2C_TRAP
    toolBar.down( '#addSnmpV2cTrapProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[ 0 ].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.model.message.channel.MessageChannel.SNMPV2C_TRAP
        } ).execute();
      }
    }, me );

    // SNMPV3_TRAP
    toolBar.down( '#addSnmpV3TrapProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[ 0 ].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.model.message.channel.MessageChannel.SNMPV3_TRAP
        } ).execute();
      }
    }, me );


    // WEB_SERVICE
    toolBar.down( '#addWebServiceProfileButton' ).on( 'click', function ()
    {
      var selection = treePanel.getSelectionModel().getSelection();
      if ( !Ext.isEmpty( selection ) )
      {
        var folderId = selection[ 0 ].get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
          maskEl : window.getEl(),
          parentFolderId : folderId,
          messageChannel : QDesk.model.message.channel.MessageChannel.WEB_SERVICE
        } ).execute();
      }
    }, me );
  },


  /**
   * Invoked when the refreshButton in the message channel profiles tree panel is clicked.
   */
  _onRefreshButtonClick : function ()
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder', {} ).execute();
  },


  /**
   * Invoked when the collapseAllButton in the message channel profiles tree panel is clicked.
   */
  _onCollapseAllButtonClick : function ()
  {
    var me = this;
    var treePanel = QDesk.util.MessageChannelProfileUtils.getProfilesTreePanel();
    treePanel.collapseAll();
  },


  /**
   * Invoked when the deleteFolderOrProfileButton in the message channel profiles tree panel is clicked.
   */
  _onDeleteFolderOrProfileButtonClick : function ()
  {
    var me = this;

    var window = QDesk.util.MessageChannelProfileUtils.getProfilesWindow();

    var selectedRecord = me._getSelectedTreeRecord();
    if ( selectedRecord )
    {
      var nodeId = selectedRecord.get( 'id' );
      var nodeType = selectedRecord.get( 'nodeType' );
      if ( nodeType == 'folder' )
      {
        var folderId = selectedRecord.get( 'folderId' );
        Ext.create( 'QDesk.controller.command.message.channel.DeleteMessageChannelProfileFolder', {
          maskEl : window.getEl(),
          folderId : folderId
        } ).execute();
      }
      else if ( nodeType == 'profile' )
      {
        var profileId = selectedRecord.get( 'profileId' );
        Ext.create( 'QDesk.controller.command.message.channel.DeleteMessageChannelProfile', {
          maskEl : window.getEl(),
          profileId : profileId
        } ).execute();
      }
    }
  },


  /**
   * Invoked when a folder is selected in the message channel profiles tree panel.
   * @param folderRecord a folder record.
   */
  _onFolderSelected : function ( folderRecord )
  {
    var me = this;

    var treePanel = QDesk.util.MessageChannelProfileUtils.getProfilesTreePanel();
    var toolBar = treePanel.down( '#toolBar' );

    var addButton = toolBar.down( '#addFolderOrProfileButton' );
    var deleteButton = toolBar.down( '#deleteFolderOrProfileButton' );

    // enable add action
    addButton.enable();

    // delete button tooltips differ based on whether a folder, or profile is selected
    deleteButton.setTooltip(
        QDesk.Msg.get(
            'QDesk.controller.command.message.channel.EditMessageChannelProfiles.tooltipDeleteFolderButton' ) );

    // delete button enabled only for non-root folders
    var nodeId = folderRecord.get( 'id' );
    if ( nodeId == QDesk.util.constant.Const.MESSAGE_CHANNEL_PROFILES_ROOT_ID )
    {
      deleteButton.disable();

      QDesk.util.MessageChannelProfileUtils.cleanupEditPanel();
    }
    else
    {
      deleteButton.enable();

      var window = QDesk.util.MessageChannelProfileUtils.getProfilesWindow();

      Ext.create( 'QDesk.controller.command.message.channel.EditMessageChannelProfileFolder', {
        maskEl : window.getEl(),
        folder : folderRecord
      } ).execute();
    }
  },


  /**
   * Invoked when a profile is selected in the message channel profiles tree panel.
   * @param profileRecord a profile record.
   */
  _onProfileSelected : function ( profileRecord )
  {
    var me = this;

    var treePanel = QDesk.util.MessageChannelProfileUtils.getProfilesTreePanel();
    var toolBar = treePanel.down( '#toolBar' );

    var addButton = toolBar.down( '#addFolderOrProfileButton' );
    var deleteButton = toolBar.down( '#deleteFolderOrProfileButton' );

    // disable add action
    addButton.disable();

    // delete button tooltips differ based on whether a folder, or profile is selected
    deleteButton.setTooltip(
        QDesk.Msg.get(
            'QDesk.controller.command.message.channel.EditMessageChannelProfiles.tooltipDeleteProfileButton' ) );

    deleteButton.enable();

    var window = QDesk.util.MessageChannelProfileUtils.getProfilesWindow();

    var profileId = profileRecord.get( 'profileId' );
    var messageChannel = profileRecord.get( 'messageChannel' );

    Ext.create( 'QDesk.controller.command.message.channel.EditMessageChannelProfile', {
      maskEl : window.getEl(),
      profileId : profileId,
      messageChannel : messageChannel
    } ).execute();
  },


  /**
   * Invoked when a profile, or folder is unselected in the message channel profiles tree panel.
   */
  _onUnselected : function ()
  {
    var me = this;

    var treePanel = QDesk.util.MessageChannelProfileUtils.getProfilesTreePanel();
    var toolBar = treePanel.down( '#toolBar' );

    var addButton = toolBar.down( '#addFolderOrProfileButton' );
    var deleteButton = toolBar.down( '#deleteFolderOrProfileButton' );

    // disable add action
    addButton.disable();

    // disable delete button and reset its tooltips (tooltips change dynamically)
    deleteButton.setTooltip( '' );
    deleteButton.disable();

    //QDesk.util.MessageChannelProfileUtils.cleanupEditPanel();
  },


  /**
   * Creates and returns the context menu for a folder node in the message channel profile tree.
   * @param folderRecord a folder record (see QDesk.model.MessageChannelProfilesTreeNode}.
   * @return the created context menu.
   */
  _createFolderCtxMenu : function ( folderRecord )
  {
    var me = this;

    var window = QDesk.util.MessageChannelProfileUtils.getProfilesWindow();

    var nodeId = folderRecord.get( 'id' );
    var folderId = folderRecord.get( 'folderId' );
    var isRootFolder = nodeId == QDesk.util.constant.Const.MESSAGE_CHANNEL_PROFILES_ROOT_ID;

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        {
          text : QDesk.Msg.get(
              'QDesk.controller.command.message.channel.EditMessageChannelProfiles.folderCtxMenuAddFolderOrProfile' ),
          qdRequiresAnyPermissions : [
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_FLD_ADD,
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD
          ],
          menu : {
            items : [
              {
                text : QDesk.Msg.get(
                    'QDesk.controller.command.message.channel.EditMessageChannelProfiles.folderCtxMenuAddFolder' ),
                iconCls : 'addFolder',
                qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_FLD_ADD ],
                qdNotAuthorizedIconCls : 'addFolderNAUTH',
                handler : function ()
                {
                  Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfileFolder', {
                    maskEl : window.getEl(),
                    parentFolderId : folderId
                  } ).execute();
                }
              },
              '-',
              {
                text : QDesk.util.EnumUtils.getMessageChannelName( QDesk.model.message.channel.MessageChannel.EMAIL ),
                iconCls : QDesk.model.message.channel.MessageChannel.EMAIL,
                qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                qdNotAuthorizedIconCls : QDesk.model.message.channel.MessageChannel.EMAIL + '_NAUTH',
                handler : function ()
                {
                  Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                    maskEl : window.getEl(),
                    parentFolderId : folderId,
                    messageChannel : QDesk.model.message.channel.MessageChannel.EMAIL
                  } ).execute();
                }
              },

              /*
               * Collaboration message channel profiles.
               */
              {
                text : QDesk.Msg.get(
                    'QDesk.controller.command.message.channel.EditMessageChannelProfiles.folderCtxMenuAddCollaborationProfileGroup' ),
                iconCls : 'addCollaborationProfile',
                qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                qdNotAuthorizedIconCls : 'addCollaborationProfileNAUTH',
                menu : [
                  {
                    text : QDesk.util.EnumUtils.getMessageChannelName(
                        QDesk.model.message.channel.MessageChannel.COLLABORATION_HIPCHAT ),
                    iconCls : QDesk.model.message.channel.MessageChannel.COLLABORATION_HIPCHAT,
                    qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                    qdNotAuthorizedIconCls : QDesk.model.message.channel.MessageChannel.COLLABORATION_HIPCHAT +
                    '_NAUTH',
                    handler : function ()
                    {
                      Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                        maskEl : window.getEl(),
                        parentFolderId : folderId,
                        messageChannel : QDesk.model.message.channel.MessageChannel.COLLABORATION_HIPCHAT
                      } ).execute();
                    }
                  },
                  {
                    text : QDesk.util.EnumUtils.getMessageChannelName(
                        QDesk.model.message.channel.MessageChannel.COLLABORATION_SLACK ),
                    iconCls : QDesk.model.message.channel.MessageChannel.COLLABORATION_SLACK,
                    qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                    qdNotAuthorizedIconCls : QDesk.model.message.channel.MessageChannel.COLLABORATION_SLACK + '_NAUTH',
                    handler : function ()
                    {
                      Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                        maskEl : window.getEl(),
                        parentFolderId : folderId,
                        messageChannel : QDesk.model.message.channel.MessageChannel.COLLABORATION_SLACK
                      } ).execute();
                    }
                  }
                ]
              },

              /*
               * Instant messaging message channel profiles.
               */
              {
                text : QDesk.Msg.get(
                    'QDesk.controller.command.message.channel.EditMessageChannelProfiles.folderCtxMenuAddInstantMessageProfileGroup' ),
                iconCls : 'addInstantMessageProfile',
                qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                qdNotAuthorizedIconCls : 'addInstantMessageProfileNAUTH',
                menu : [
                  {
                    text : QDesk.util.EnumUtils.getMessageChannelName(
                        QDesk.model.message.channel.MessageChannel.IM_AIM ),
                    iconCls : QDesk.model.message.channel.MessageChannel.IM_AIM,
                    qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                    qdNotAuthorizedIconCls : QDesk.model.message.channel.MessageChannel.IM_AIM + '_NAUTH',
                    handler : function ()
                    {
                      Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                        maskEl : window.getEl(),
                        parentFolderId : folderId,
                        messageChannel : QDesk.model.message.channel.MessageChannel.IM_AIM
                      } ).execute();
                    }
                  },
                  {
                    text : QDesk.util.EnumUtils.getMessageChannelName(
                        QDesk.model.message.channel.MessageChannel.IM_GTALK ),
                    iconCls : QDesk.model.message.channel.MessageChannel.IM_GTALK,
                    qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                    qdNotAuthorizedIconCls : QDesk.model.message.channel.MessageChannel.IM_GTALK + '_NAUTH',
                    handler : function ()
                    {
                      Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                        maskEl : window.getEl(),
                        parentFolderId : folderId,
                        messageChannel : QDesk.model.message.channel.MessageChannel.IM_GTALK
                      } ).execute();
                    }
                  },
                  {
                    text : QDesk.util.EnumUtils.getMessageChannelName(
                        QDesk.model.message.channel.MessageChannel.IM_ICQ ),
                    iconCls : QDesk.model.message.channel.MessageChannel.IM_ICQ,
                    qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                    qdNotAuthorizedIconCls : QDesk.model.message.channel.MessageChannel.IM_ICQ + '_NAUTH',
                    handler : function ()
                    {
                      Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                        maskEl : window.getEl(),
                        parentFolderId : folderId,
                        messageChannel : QDesk.model.message.channel.MessageChannel.IM_ICQ
                      } ).execute();
                    }
                  },
                  {
                    text : QDesk.util.EnumUtils.getMessageChannelName(
                        QDesk.model.message.channel.MessageChannel.IM_XMPP ),
                    iconCls : QDesk.model.message.channel.MessageChannel.IM_XMPP,
                    qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                    qdNotAuthorizedIconCls : QDesk.model.message.channel.MessageChannel.IM_XMPP + '_NAUTH',
                    handler : function ()
                    {
                      Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                        maskEl : window.getEl(),
                        parentFolderId : folderId,
                        messageChannel : QDesk.model.message.channel.MessageChannel.IM_XMPP
                      } ).execute();
                    }
                  },
                  {
                    text : QDesk.util.EnumUtils.getMessageChannelName(
                        QDesk.model.message.channel.MessageChannel.IM_YAHOO ),
                    iconCls : QDesk.model.message.channel.MessageChannel.IM_YAHOO,
                    qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                    qdNotAuthorizedIconCls : QDesk.model.message.channel.MessageChannel.IM_YAHOO + '_NAUTH',
                    handler : function ()
                    {
                      Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                        maskEl : window.getEl(),
                        parentFolderId : folderId,
                        messageChannel : QDesk.model.message.channel.MessageChannel.IM_YAHOO
                      } ).execute();
                    }
                  }
                ]
              },

              /*
               * SMS message channel profiles.
               */
              {
                text : QDesk.Msg.get(
                    'QDesk.controller.command.message.channel.EditMessageChannelProfiles.folderCtxMenuAddSmsProfileGroup' ),
                iconCls : 'addSmsProfile',
                qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                qdNotAuthorizedIconCls : 'addSmsProfileNAUTH',
                menu : [
                  {
                    text : QDesk.util.EnumUtils.getMessageChannelName(
                        QDesk.model.message.channel.MessageChannel.SMS_CLICKATELL_CENTRAL ),
                    iconCls : QDesk.model.message.channel.MessageChannel.SMS_CLICKATELL_CENTRAL,
                    qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                    qdNotAuthorizedIconCls : QDesk.model.message.channel.MessageChannel.SMS_CLICKATELL_CENTRAL +
                    '_NAUTH',
                    handler : function ()
                    {
                      Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                        maskEl : window.getEl(),
                        parentFolderId : folderId,
                        messageChannel : QDesk.model.message.channel.MessageChannel.SMS_CLICKATELL_CENTRAL
                      } ).execute();
                    }
                  },
                  {
                    text : QDesk.util.EnumUtils.getMessageChannelName(
                        QDesk.model.message.channel.MessageChannel.SMS_CLICKATELL_PLATFORM ),
                    iconCls : QDesk.model.message.channel.MessageChannel.SMS_CLICKATELL_PLATFORM,
                    qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                    qdNotAuthorizedIconCls : QDesk.model.message.channel.MessageChannel.SMS_CLICKATELL_PLATFORM +
                    '_NAUTH',
                    handler : function ()
                    {
                      Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                        maskEl : window.getEl(),
                        parentFolderId : folderId,
                        messageChannel : QDesk.model.message.channel.MessageChannel.SMS_CLICKATELL_PLATFORM
                      } ).execute();
                    }
                  },
                  {
                    text : QDesk.util.EnumUtils.getMessageChannelName(
                        QDesk.model.message.channel.MessageChannel.SMS_CLOCKWORK ),
                    iconCls : QDesk.model.message.channel.MessageChannel.SMS_CLOCKWORK,
                    qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                    qdNotAuthorizedIconCls : QDesk.model.message.channel.MessageChannel.SMS_CLOCKWORK + '_NAUTH',
                    handler : function ()
                    {
                      Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                        maskEl : window.getEl(),
                        parentFolderId : folderId,
                        messageChannel : QDesk.model.message.channel.MessageChannel.SMS_CLOCKWORK
                      } ).execute();
                    }
                  },
                  {
                    text : QDesk.util.EnumUtils.getMessageChannelName(
                        QDesk.model.message.channel.MessageChannel.SMS_NEXMO ),
                    iconCls : QDesk.model.message.channel.MessageChannel.SMS_NEXMO,
                    qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                    qdNotAuthorizedIconCls : QDesk.model.message.channel.MessageChannel.SMS_NEXMO + '_NAUTH',
                    handler : function ()
                    {
                      Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                        maskEl : window.getEl(),
                        parentFolderId : folderId,
                        messageChannel : QDesk.model.message.channel.MessageChannel.SMS_NEXMO
                      } ).execute();
                    }
                  },
                  {
                    text : QDesk.util.EnumUtils.getMessageChannelName(
                        QDesk.model.message.channel.MessageChannel.SMS_SMSGLOBAL ),
                    iconCls : QDesk.model.message.channel.MessageChannel.SMS_SMSGLOBAL,
                    qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                    qdNotAuthorizedIconCls : QDesk.model.message.channel.MessageChannel.SMS_SMSGLOBAL + '_NAUTH',
                    handler : function ()
                    {
                      Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                        maskEl : window.getEl(),
                        parentFolderId : folderId,
                        messageChannel : QDesk.model.message.channel.MessageChannel.SMS_SMSGLOBAL
                      } ).execute();
                    }
                  },
                  {
                    text : QDesk.util.EnumUtils.getMessageChannelName(
                        QDesk.model.message.channel.MessageChannel.SMS_TWILIO ),
                    iconCls : QDesk.model.message.channel.MessageChannel.SMS_TWILIO,
                    qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                    qdNotAuthorizedIconCls : QDesk.model.message.channel.MessageChannel.SMS_TWILIO + '_NAUTH',
                    handler : function ()
                    {
                      Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                        maskEl : window.getEl(),
                        parentFolderId : folderId,
                        messageChannel : QDesk.model.message.channel.MessageChannel.SMS_TWILIO
                      } ).execute();
                    }
                  }
                ]
              },

              /*
               * SNMP message channel profiles.
               */
              {
                text : QDesk.Msg.get(
                    'QDesk.controller.command.message.channel.EditMessageChannelProfiles.folderCtxMenuAddSnmpProfileGroup' ),
                iconCls : 'addSnmpProfile',
                qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                qdNotAuthorizedIconCls : 'addSnmpProfileNAUTH',
                menu : [
                  {
                    text : QDesk.util.EnumUtils.getMessageChannelName(
                        QDesk.model.message.channel.MessageChannel.SNMPV1_TRAP ),
                    iconCls : QDesk.model.message.channel.MessageChannel.SNMPV1_TRAP,
                    qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                    qdNotAuthorizedIconCls : QDesk.model.message.channel.MessageChannel.SNMPV1_TRAP + '_NAUTH',
                    handler : function ()
                    {
                      Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                        maskEl : window.getEl(),
                        parentFolderId : folderId,
                        messageChannel : QDesk.model.message.channel.MessageChannel.SNMPV1_TRAP
                      } ).execute();
                    }
                  },
                  {
                    text : QDesk.util.EnumUtils.getMessageChannelName(
                        QDesk.model.message.channel.MessageChannel.SNMPV2C_TRAP ),
                    iconCls : QDesk.model.message.channel.MessageChannel.SNMPV2C_TRAP,
                    qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                    qdNotAuthorizedIconCls : QDesk.model.message.channel.MessageChannel.SNMPV2C_TRAP + '_NAUTH',
                    handler : function ()
                    {
                      Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                        maskEl : window.getEl(),
                        parentFolderId : folderId,
                        messageChannel : QDesk.model.message.channel.MessageChannel.SNMPV2C_TRAP
                      } ).execute();
                    }
                  },
                  {
                    text : QDesk.util.EnumUtils.getMessageChannelName(
                        QDesk.model.message.channel.MessageChannel.SNMPV3_TRAP ),
                    iconCls : QDesk.model.message.channel.MessageChannel.SNMPV3_TRAP,
                    qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                    qdNotAuthorizedIconCls : QDesk.model.message.channel.MessageChannel.SNMPV3_TRAP + '_NAUTH',
                    handler : function ()
                    {
                      Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                        maskEl : window.getEl(),
                        parentFolderId : folderId,
                        messageChannel : QDesk.model.message.channel.MessageChannel.SNMPV3_TRAP
                      } ).execute();
                    }
                  }
                ]
              },

              {
                text : QDesk.util.EnumUtils.getMessageChannelName(
                    QDesk.model.message.channel.MessageChannel.WEB_SERVICE ),
                iconCls : QDesk.model.message.channel.MessageChannel.WEB_SERVICE,
                qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD ],
                qdNotAuthorizedIconCls : QDesk.model.message.channel.MessageChannel.WEB_SERVICE + '_NAUTH',
                handler : function ()
                {
                  Ext.create( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
                    maskEl : window.getEl(),
                    parentFolderId : folderId,
                    messageChannel : QDesk.model.message.channel.MessageChannel.WEB_SERVICE
                  } ).execute();
                }
              }
            ]
          }
        },
        {
          text : QDesk.Msg.get(
              'QDesk.controller.command.message.channel.EditMessageChannelProfiles.folderCtxMenuDelete' ),
          iconCls : 'deleteFolder',
          disabled : isRootFolder,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_FLD_DELETE ],
          qdNotAuthorizedIconCls : 'deleteFolderNAUTH',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.message.channel.DeleteMessageChannelProfileFolder', {
              maskEl : window.getEl(),
              folderId : folderId
            } ).execute();
          }
        },
        {
          text : QDesk.Msg.get(
              'QDesk.controller.command.message.channel.EditMessageChannelProfiles.folderCtxMenuRefresh' ),
          iconCls : 'refreshFolder',
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'refreshFolderNAUTH',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder', {
              maskEl : window.getEl(),
              folderId : folderId
            } ).execute();
          }
        }
      ]
    } );
  },


  /**
   * Creates and returns the context menu for a profile node in the message channel profile tree.
   * @param profileRecord a profile record (see QDesk.model.MessageChannelProfilesTreeNode}.
   * @return the created context menu.
   */
  _createProfileCtxMenu : function ( profileRecord )
  {
    var me = this;

    var window = QDesk.util.MessageChannelProfileUtils.getProfilesWindow();

    var profileId = profileRecord.get( 'profileId' );
    var messageChannel = profileRecord.get( 'messageChannel' );

    return Ext.create( 'Ext.menu.Menu', {
      items : [
//
//        // edit profile
//        {
//          text: QDesk.Msg.get( 'QDesk.controller.command.message.channel.EditMessageChannelProfiles.profileCtxMenuEdit' ),
//          iconCls: 'editProfile',
//          handler: function ()
//          {
//            Ext.create( 'QDesk.controller.command.message.channel.EditMessageChannelProfile', {
//              maskEl: window.getEl(),
//              profileId: profileId,
//              messageChannel: messageChannel
//            } ).execute();
//          }
//        },
        // delete profile
        {
          text : QDesk.Msg.get(
              'QDesk.controller.command.message.channel.EditMessageChannelProfiles.profileCtxMenuDelete' ),
          iconCls : 'deleteProfile',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_DELETE ],
          qdNotAuthorizedIconCls : 'deleteProfileNAUTH',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.message.channel.DeleteMessageChannelProfile', {
              maskEl : window.getEl(),
              profileId : profileId
            } ).execute();
          }
        }
      ]
    } );
  },


  /**
   * Returns the currently selected record (a {@link QDesk.model.message.MessageChannelProfilesTreeNode} instance),
   * or null if no record is selected.
   */
  _getSelectedTreeRecord : function ()
  {
    var me = this;

    var treePanel = QDesk.util.MessageChannelProfileUtils.getProfilesTreePanel();
    var selectedRecords = treePanel.getSelectionModel().getSelection();
    if ( Ext.isEmpty( selectedRecords ) )
      return null;
    else
      return selectedRecords[ 0 ];
  },


  _onNodeDrag : function ( draggedRecord, targetRecord )
  {
    var me = this;

    var window = QDesk.util.MessageChannelProfileUtils.getProfilesWindow();

    var targetFolderId = targetRecord.get( 'folderId' );
    var nodeType = draggedRecord.get( 'nodeType' );
    if ( nodeType == 'folder' )
    {
      var folderId = draggedRecord.get( 'folderId' );
      Ext.create( 'QDesk.controller.command.message.channel.MoveMessageChannelProfileFolder', {
        maskEl : window.getEl(),
        folderId : folderId,
        targetFolderId : targetFolderId
      } ).execute();
    }
    else if ( nodeType == 'profile' )
    {
      var profileId = draggedRecord.get( 'profileId' );
      Ext.create( 'QDesk.controller.command.message.channel.MoveMessageChannelProfile', {
        maskEl : window.getEl(),
        profileId : profileId,
        targetFolderId : targetFolderId
      } ).execute();
    }
  },


  /**
   * Checks if the specified record can be dragged to the specified target record.
   * @param draggedRecord the dragged folder/profile record.
   * @param targetRecord the target record (target folder).
   * @return {boolean} true if the dragged folder/profile can be moved, false otherwise.
   * @private
   */
  _checkNodeMove : function ( draggedRecord, targetRecord )
  {
    var canBeMoved = true;

    var targetFolderId = targetRecord.get( 'folderId' );
    var targetFolderName = targetRecord.get( 'name' );

    var nodeType = draggedRecord.get( 'nodeType' );
    if ( nodeType == 'folder' )
    {
      if ( QDesk.CurrentUser.hasAnyPermissions( [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_FLD_MOVE ] ) )
      {
        var folderId = draggedRecord.get( 'folderId' );
        var folderName = draggedRecord.get( 'name' );

        Ext.each( targetRecord.childNodes, function ( node )
        {
          if ( node.get( 'nodeType' ) == 'folder' && node.get( 'name' ) == folderName )
          {
            QDesk.util.UxUtils.showWarningDialog( QDesk.Msg.get(
                'QDesk.controller.command.message.channel.EditMessageChannelProfiles.folderCannotBeMovedMsg',
                Ext.String.htmlEncode( folderName ), Ext.String.htmlEncode( targetFolderName ) ) );

            canBeMoved = false;
            return false;
          }
        } );
      }
      else
      {
        QDesk.util.UxUtils.showWarningDialog(
            QDesk.Msg.get(
                'QDesk.controller.command.message.channel.EditMessageChannelProfiles.notEnoughPermissionsFolderMoveMsg' ) );
        canBeMoved = false;
      }
    }
    else if ( nodeType == 'profile' )
    {
      if ( QDesk.CurrentUser.hasAnyPermissions( [ QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_MOVE ] ) )
      {
        var profileId = draggedRecord.get( 'profileId' );
        var profileName = draggedRecord.get( 'name' );

        Ext.each( targetRecord.childNodes, function ( node )
        {
          if ( node.get( 'nodeType' ) == 'profile' && node.get( 'name' ) == profileName )
          {
            QDesk.util.UxUtils.showWarningDialog( QDesk.Msg.get(
                'QDesk.controller.command.message.channel.EditMessageChannelProfiles.profileCannotBeMovedMsg',
                Ext.String.htmlEncode( profileName ), Ext.String.htmlEncode( targetFolderName ) ) );

            canBeMoved = false;
            return false;
          }
        } );
      }
      else
      {
        QDesk.util.UxUtils.showWarningDialog(
            QDesk.Msg.get(
                'QDesk.controller.command.message.channel.EditMessageChannelProfiles.notEnoughPermissionsProfileMoveMsg' ) );
        canBeMoved = false;
      }
    }

    return canBeMoved;
  }
} );
