/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Stops the scheduler whose connection ID is specified in the configuration parameters.
 */
Ext.define( 'QDesk.controller.command.scheduler.common.StopAllSchedulers', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.model.gui.DialogType',
    'QDesk.model.common.Result',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.view.work.scheduler.common.ConnectionsOperationResultsWindow',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.EventName'
  ],

  /*
   * Required ID of the folder containing schedulers to stop.
   */
  folderId : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.folderId )
      Ext.Error.raise( 'Required config parameter missing: folderId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'confirm.schedulersStopAll' ) )
    {
      Ext.MessageBox.show( {
        title : QDesk.Msg.get( 'QDesk.util.UxUtils.dialogTitleConfirm' ),

        msg : QDesk.Msg.get( 'QDesk.controller.command.scheduler.common.StopAllSchedulers.confirmMsg',
            QDesk.controller.command.connection.ConnectionUtils.getFolderName( me.folderId ) ),

        buttons : Ext.MessageBox.YESNOCANCEL,
        buttonText : {
          yes : QDesk.Msg.get( 'QDesk.buttonLabelYes' ),
          no : QDesk.Msg.get( 'QDesk.buttonLabelYesIncludeSubfolders' ),
          cancel : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
        },

        icon : Ext.MessageBox.QUESTION,

        fn : function ( btn )
        {
          if ( btn == 'yes' )
          {
            me._stopSchedulers( false );
          }
          else if ( btn == 'no' )
          {
            me._stopSchedulers( true );
          }
        }
      } );
    }
    else
    {
      me._stopSchedulers( true );
    }
  },


  _stopSchedulers : function ( recursive )
  {
    var me = this;

    var folderName = QDesk.controller.command.connection.ConnectionUtils.getFolderName( me.folderId );

    QDesk.util.UxUtils.maskBody(
        QDesk.Msg.get( recursive ?
            'QDesk.controller.command.scheduler.common.StopAllSchedulers.maskRecursiveMsg' :
            'QDesk.controller.command.scheduler.common.StopAllSchedulers.maskNonRecursiveMsg',
            Ext.String.htmlEncode( folderName ) ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.connectionManagementActionStopAll,
      params : {
        folderId : me.folderId,
        recursive : recursive
      },
      callback : function ( opts, success, resp )
      {
        QDesk.util.UxUtils.unmaskBody();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            if ( Ext.isEmpty( json.data ) )
            {
              // no schedulers found in the folder
              QDesk.util.UxUtils.showWarningDialog( QDesk.Msg.get(
                  recursive ?
                      'QDesk.controller.command.scheduler.common.StopAllSchedulers.warnNoSchedulersFoundRecursiveMsg' :
                      'QDesk.controller.command.scheduler.common.StopAllSchedulers.warnNoSchedulersFoundNonRecursiveMsg',
                  Ext.String.htmlEncode( folderName ) ) );
            }
            else
            {
              var successItems = [];
              var errorItems = [];

              // now we need to check the json.result list to see which schedulers have been stopped and which have not
              Ext.each( json.data, function ( item )
              {
                if ( item.result == QDesk.model.common.Result.SUCCESS )
                {
                  successItems.push( item );
                }
                else if ( item.result == QDesk.model.common.Result.ERROR )
                {
                  errorItems.push( item );
                }
              } );

              if ( Ext.isEmpty( errorItems ) )
              {
                // no errors - all schedulers have been stopped
                QDesk.util.UxUtils.showInfo( QDesk.Msg.get( recursive ?
                    'QDesk.controller.command.scheduler.common.StopAllSchedulers.successRecursiveMsg' :
                    'QDesk.controller.command.scheduler.common.StopAllSchedulers.successNonRecursiveMsg',
                    Ext.String.htmlEncode( folderName ) ),
                    function ()
                    {
                      Ext.each( successItems, function ( item )
                      {
                        me.application.fireEvent( QDesk.util.constant.EventName.SCHEDULER_STOPPED, {
                          connectionId : item.connectionId
                        } );
                      } );
                    } );
              }
              else
              {
                // some of the schedulers in the folder or its sub-folders were not stopped
                var window = Ext.widget( 'ConnectionsOperationResultsWindow', {
                  qdesk : {
                    dialogType : QDesk.model.gui.DialogType.WARNING,
                    message : QDesk.Msg.get( recursive ?
                        'QDesk.controller.command.scheduler.common.StopAllSchedulers.warnSomeSchedulersNotStoppedRecursiveMsg' :
                        'QDesk.controller.command.scheduler.common.StopAllSchedulers.warnSomeSchedulersNotStoppedNonRecursiveMsg',
                        Ext.String.htmlEncode( folderName ) ),
                    results : json.data,
                    callback : function ()
                    {
                      Ext.each( successItems, function ( item )
                      {
                        me.application.fireEvent( QDesk.util.constant.EventName.SCHEDULER_STOPPED, {
                          connectionId : item.connectionId
                        } );
                      } );
                    }
                  }
                } );

                window.show();
              }
            }
          }
          else
          {
            // json.success = false
            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        }
        else
        {
          // Ajax error
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.common.StopAllSchedulers.errorMsg',
                  Ext.String.htmlEncode( folderName ) ) );
        }

        // reload the folder in the connections tree view
        Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
          folderId : me.folderId
        } ).execute();
      }
    } );
  }
} );
