/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the Scheduler Job Chains tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.chain.InitializeSchedulerJobChainsTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.connection.ConnectionUtils',

    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',

    'QDesk.controller.command.scheduler.quartz.chain.AddJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.CloneJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.DeleteJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.DisableJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.EditJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.EnableJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsGrid',
    'QDesk.controller.command.scheduler.quartz.chain.ViewJobChainAuditTrail',

    'QDesk.util.UxUtils',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var schedulerJobChainsTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getSchedulerJobChainsTab(
        me.connectionId );

    me._bindEventHandlers( schedulerJobChainsTab );

    schedulerJobChainsTab.setInitialized( true );
  },


  /**
   * Binds events of the specified scheduler Job Chains tab to their respective handlers.
   */
  _bindEventHandlers : function ( schedulerJobChainsTab )
  {
    var me = this;
    me._bindEventHandlersJobChainsGrid( schedulerJobChainsTab );
  },


  /**
   * Binds the Job Chains grid events to their handlers.
   */
  _bindEventHandlersJobChainsGrid : function ( schedulerJobChainsTab )
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var jvmAgentReady = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentReady( me.connectionId );
    //var publicApiReady = QDesk.controller.command.connection.ConnectionUtils.isPublicApiReady( me.connectionId );

    if ( licensedFeatures.JobChaining && jvmAgentReady )
    {
      var jobChainsGrid = schedulerJobChainsTab.down( '#jobChainsGrid' );

      // refresh button
      jobChainsGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshJobChainsGrid( me.connectionId );
          } );

      jobChainsGrid.on( 'selectionchange', function ( selectionModel, selectedRecords )
      {
        if ( selectedRecords.length == 0 )
        {
          me._onJobChainUnselected( schedulerJobChainsTab );
        }
        else
        {
          me._onJobChainSelected( selectedRecords[ 0 ], schedulerJobChainsTab );
        }
      } );

      var filterQueryText = jobChainsGrid.down( '#filterQueryText' );

      var jobChainsIndexEnabled =
          QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzJobChainsIndexEnabled( me.connectionId );

      if ( !jobChainsIndexEnabled )
      {
        filterQueryText.setEmptyText(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.chain.InitializeSchedulerJobChainsTab.jobChainsIndexDisabledEmptyText' ) );
      }

      jobChainsGrid.getStore().on( 'beforeload', function ( store, operation )
      {
        jobChainsGrid.down( '#refreshButton' ).disable();
        jobChainsGrid.down( '#addButton' ).disable();
        jobChainsGrid.down( '#deleteButton' ).disable();
        jobChainsGrid.down( '#editButton' ).disable();
        jobChainsGrid.down( '#cloneButton' ).disable();
        jobChainsGrid.down( '#viewAuditTrailButton' ).disable();
        filterQueryText.disable();

        /*
         * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
         * Therefore we need to force-fire this event by clearing the selection.
         */
        jobChainsGrid.getSelectionModel().deselectAll();
      } );

      jobChainsGrid.getStore().on( 'load', function ( store, records, successful, operation )
      {
        jobChainsGrid.down( '#refreshButton' ).enable();

        // enable only if the job chains index is enabled in the agent
        if ( jobChainsIndexEnabled )
        {
          filterQueryText.enable();
        }

        if ( successful )
        {
          jobChainsGrid.down( '#addButton' ).enable();
        }
        else
        {
          store.removeAll();

          var response = operation.getResponse();
          if ( response && response.responseText )
          {
            // application errors
            QDesk.util.UxUtils.showJsonResponseErrors( Ext.JSON.decode( response.responseText ) );
          }
          else
          {
            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get(
                    'QDesk.controller.command.scheduler.quartz.chain.InitializeSchedulerJobChainsTab.errorLoadingJobChainsMsg' ) );
          }
        }
      } );

      jobChainsGrid.on( 'addJobChain',
          function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.AddJobChain', {
              connectionId : me.connectionId,
              /*
               * Upon success refresh the job chains tab.
               */
              success : function ()
              {
                Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsTab', {
                  connectionId : me.connectionId
                } ).execute();
              }
            } ).execute();
          } );

      jobChainsGrid.on( 'enableJobChain',
          function ( jobChain )
          {
            me._enableJobChain( jobChain );
          } );

      jobChainsGrid.on( 'disableJobChain',
          function ( jobChain )
          {
            me._disableJobChain( jobChain );
          } );

      jobChainsGrid.on( 'deleteJobChain',
          function ( jobChain )
          {
            me._deleteJobChain( jobChain );
          } );

      jobChainsGrid.on( 'editJobChain',
          function ( jobChain )
          {
            me._editJobChain( jobChain );
          } );

      jobChainsGrid.on( 'cloneJobChain',
          function ( jobChain )
          {
            me._cloneJobChain( jobChain );
          } );

      jobChainsGrid.on( 'viewAuditTrail',
          function ( jobChain )
          {
            me._viewJobChainAuditTrail( jobChain );
          } );
    }
  },


  /**
   * Invoked when a job chain record in the SchedulerJobChainsGrid in the SchedulerJobChainsTab
   * is selected.
   * @param jobChain a QDesk.model.scheduler.quartz.QuartzJobChain instance representing the selected
   * job chain.
   * @param schedulerJobChainsTab the scheduler Job Chains tab.
   */
  _onJobChainSelected : function ( jobChain, schedulerJobChainsTab )
  {
    var me = this;

    var jobChainsGrid = schedulerJobChainsTab.down( '#jobChainsGrid' );

    jobChainsGrid.down( '#deleteButton' ).enable();
    jobChainsGrid.down( '#editButton' ).enable();
    jobChainsGrid.down( '#cloneButton' ).enable();
    jobChainsGrid.down( '#viewAuditTrailButton' ).enable();
  },


  /**
   * Invoked when a job chain record in the SchedulerJobChainsGrid in the SchedulerJobChainsTab
   * is unselected.
   * @param schedulerJobChainsTab the scheduler Job Chains tab.
   */
  _onJobChainUnselected : function ( schedulerJobChainsTab )
  {
    var me = this;

    var jobChainsGrid = schedulerJobChainsTab.down( '#jobChainsGrid' );

    jobChainsGrid.down( '#deleteButton' ).disable();
    jobChainsGrid.down( '#editButton' ).disable();
    jobChainsGrid.down( '#cloneButton' ).disable();
    jobChainsGrid.down( '#viewAuditTrailButton' ).disable();
  },


  /**
   * Invoked to refresh the data in the scheduler job chains grid.
   * @param connectionId a connection ID.
   */
  _refreshJobChainsGrid : function ( connectionId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsGrid', {
      connectionId : connectionId
    } ).execute();
  },


  _enableJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.EnableJobChain', {
      connectionId : me.connectionId,
      jobChain : jobChain,
      /*
       * Upon success refresh the job chains tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _disableJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.DisableJobChain', {
      connectionId : me.connectionId,
      jobChain : jobChain,
      /*
       * Upon success refresh the job chains tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _deleteJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.DeleteJobChain', {
      connectionId : me.connectionId,
      jobChain : jobChain,
      /*
       * Upon success refresh the job chains tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _editJobChain : function ( jobChains )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.EditJobChain', {
      connectionId : me.connectionId,
      jobChainId : jobChains.get( 'id' ),
      /*
       * Upon success refresh the job chains tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _cloneJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.CloneJobChain', {
      connectionId : me.connectionId,
      jobChainId : jobChain.get( 'id' ),
      /*
       * Upon success refresh the job chains tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _viewJobChainAuditTrail : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.ViewJobChainAuditTrail', {
      connectionId : me.connectionId,
      jobChainId : jobChain.get( 'id' )
    } ).execute();
  }
} );
