/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the Scheduler Execution History tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.history.InitializeSchedulerExecHistoryTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',

    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',
    'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryGrid',
    'QDesk.controller.command.scheduler.quartz.history.ExportExecHistory',

    'QDesk.util.UxUtils',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var schedulerExecHistoryTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getSchedulerExecHistoryTab(
        me.connectionId );

    me._bindEventHandlers( schedulerExecHistoryTab );

    schedulerExecHistoryTab.setInitialized( true );
  },


  /**
   * Binds events of the specified scheduler Execution History tab to their respective handlers.
   */
  _bindEventHandlers : function ( schedulerExecHistoryTab )
  {
    var me = this;
    me._bindEventHandlersExecHistoryGrid( schedulerExecHistoryTab );
  },


  /**
   * Binds the jobs grid events to their handlers.
   */
  _bindEventHandlersExecHistoryGrid : function ( schedulerExecHistoryTab )
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var jvmAgentReady = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentReady( me.connectionId );
    //var publicApiReady = QDesk.controller.command.connection.ConnectionUtils.isPublicApiReady( me.connectionId );

    if ( licensedFeatures.ExecutionHistory && jvmAgentReady )
    {
      var execHistoryGrid = schedulerExecHistoryTab.down( '#execHistoryGrid' );

      // refresh button
      execHistoryGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshSchedulerExecHistoryGrid( me.connectionId );
          } );

      // export CSV button
      execHistoryGrid.down( '#exportCsvButton' ).on( 'click',
          function ()
          {
            me._exportExecHistory( me.connectionId );
          } );

      // filters changed
      execHistoryGrid.on( 'changeFilters',
          function ()
          {
            me._refreshSchedulerExecHistoryGrid( me.connectionId );
          } );

      // event fired by the actioncolumn in the JobExecHistoryGrid
      execHistoryGrid.down( '#jobDataMapColumn' ).on( 'requestViewJobDataMap',
          function ( jobDataMap )
          {
            me._viewJobDataMap( jobDataMap );
          } );

      execHistoryGrid.down( '#errorColumn' ).on( 'requestViewException',
          function ( exception )
          {
            me._viewException( me.connectionId, exception );
          } );

      execHistoryGrid.down( '#resultColumn' ).on( 'requestViewResult',
          function ( execHistoryId )
          {
            me._viewResult( me.connectionId, execHistoryId );
          } );

      execHistoryGrid.down( '#userDataColumn' ).on( 'requestViewUserData',
          function ( execHistoryId )
          {
            me._viewUserData( me.connectionId, execHistoryId );
          } );

      execHistoryGrid.down( '#logColumn' ).on( 'requestViewLog',
          function ( execHistoryId )
          {
            me._viewLog( me.connectionId, execHistoryId );
          } );

      var filterQueryText = execHistoryGrid.down( '#filterQueryText' );

      var execHistoryIndexEnabled =
          QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzExecHistoryIndexEnabled(
              me.connectionId );

      if ( !execHistoryIndexEnabled )
      {
        filterQueryText.setEmptyText(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.history.InitializeSchedulerExecHistoryTab.execHistoryIndexDisabledEmptyText' ) );
      }

      execHistoryGrid.getStore().on( 'beforeload', function ( store, operation )
      {
        execHistoryGrid.down( '#refreshButton' ).disable();
        execHistoryGrid.down( '#fetchPeriod' ).disable();
        execHistoryGrid.down( '#maxFetchSizeSelect' ).disable();
        filterQueryText.disable();
        execHistoryGrid.down( '#exportCsvButton' ).disable();

        /*
         * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
         * Therefore we need to force-fire this event by clearing the selection.
         */
        execHistoryGrid.getSelectionModel().deselectAll();
      } );

      execHistoryGrid.getStore().on( 'load', function ( store, records, successful, operation )
      {
        execHistoryGrid.down( '#refreshButton' ).enable();
        execHistoryGrid.down( '#fetchPeriod' ).enable();
        execHistoryGrid.down( '#maxFetchSizeSelect' ).enable();

        // enable only if the exec history index is enabled in the agent
        if ( execHistoryIndexEnabled )
        {
          filterQueryText.enable();
        }

        if ( successful )
        {
          if ( jvmAgentReady )
          {
            execHistoryGrid.down( '#exportCsvButton' ).enable();
          }
        }
        else
        {
          store.removeAll();

          var response = operation.getResponse();
          if ( response && response.responseText )
          {
            // application errors
            QDesk.util.UxUtils.showJsonResponseErrors( Ext.JSON.decode( response.responseText ) );
          }
          else
          {
            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get(
                    'QDesk.controller.command.scheduler.quartz.history.InitializeSchedulerExecHistoryTab.errorLoadingExecHistoryMsg' ) );
          }
        }
      } );
    }
  },


  /**
   * Invoked to refresh the data in the scheduler execution history grid.
   * @param connectionId a connection ID.
   */
  _refreshSchedulerExecHistoryGrid : function ( connectionId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryGrid', {
      connectionId : connectionId
    } ).execute();
  },


  /**
   * Opens a window for exporting scheduler execution history data to a CSV file.
   * @param connectionId a connection ID.
   * @private
   */
  _exportExecHistory : function ( connectionId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.history.ExportExecHistory', {
      connectionId : connectionId
    } ).execute();
  },


  /**
   * Opens a window showing the specified job data maps.
   * @param jobDataMap a job data map.
   */
  _viewJobDataMap : function ( jobDataMap )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ViewJobDataMap', {
      // exec history record can have null job data map !!!
      jobDataMap : jobDataMap == null ? [] : jobDataMap
    } ).execute();
  },


  /**
   * Opens a window showing the job execution exception.
   * @param connectionId a connection ID.
   * @param exception the exception data.
   */
  _viewException : function ( connectionId, exception )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.ViewError', {
      errorText : exception
    } ).execute();
  },


  /**
   * Opens a window showing the job execution log.
   * @param connectionId a connection ID.
   * @param execHistoryId an execution history ID.
   */
  _viewLog : function ( connectionId, execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryLog', {
      connectionId : connectionId,
      execHistoryId : execHistoryId
    } ).execute();
  },


  /**
   * Opens a window showing the job execution result.
   * @param connectionId a connection ID.
   * @param execHistoryId an execution history ID.
   */
  _viewResult : function ( connectionId, execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryResult', {
      connectionId : connectionId,
      execHistoryId : execHistoryId
    } ).execute();
  },


  /**
   * Opens a window showing the job execution user data.
   * @param connectionId a connection ID.
   * @param execHistoryId an execution history ID.
   */
  _viewUserData : function ( connectionId, execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryUserData', {
      connectionId : connectionId,
      execHistoryId : execHistoryId
    } ).execute();
  }
} );
