/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the Scheduler Execution Notifications tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.notif.exec.InitializeSchedulerExecNotificationsTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.connection.ConnectionUtils',

    'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator',

    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',

    'QDesk.controller.command.scheduler.quartz.notif.exec.AddExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.DeleteExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.DisableExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.EnableExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationRulesGrid',
    'QDesk.controller.command.scheduler.quartz.notif.exec.ViewExecNotificationRuleAuditTrail',

    'QDesk.model.message.MessageCreatorType',

    'QDesk.util.UxUtils',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var schedulerNotificationsTab =
        QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getSchedulerNotificationsTab( me.connectionId );

    me._bindEventHandlers( schedulerNotificationsTab );

    schedulerNotificationsTab.setInitialized( true );
  },


  /**
   * Binds events of the specified scheduler Notifications tab to their respective handlers.
   */
  _bindEventHandlers : function ( schedulerNotificationsTab )
  {
    var me = this;
    me._bindEventHandlersNotificationRulesGrid( schedulerNotificationsTab );
  },


  /**
   * Binds the notification rules grid events to their handlers.
   */
  _bindEventHandlersNotificationRulesGrid : function ( schedulerNotificationsTab )
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionNotifications &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentReady( me.connectionId ) )
    {
      var execNotificationRulesGrid = schedulerNotificationsTab.down( '#execNotificationRulesGrid' );

      // refresh button
      execNotificationRulesGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshNotificationRulesGrid( me.connectionId );
          } );

      execNotificationRulesGrid.on( 'selectionchange', function ( selectionModel, selectedRecords )
      {
        if ( selectedRecords.length == 0 )
        {
          me._onNotificationRuleUnselected( schedulerNotificationsTab );
        }
        else
        {
          me._onNotificationRuleSelected( selectedRecords[ 0 ], schedulerNotificationsTab );
        }
      } );

      var filterQueryText = execNotificationRulesGrid.down( '#filterQueryText' );

      var jvmAgentReady = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentReady( me.connectionId );

      if ( jvmAgentReady )
      {
        var notificationRulesIndexEnabled =
            QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzExecNotificationRulesIndexEnabled(
                me.connectionId );

        if ( !notificationRulesIndexEnabled )
        {
          filterQueryText.setEmptyText(
              QDesk.Msg.get(
                  'QDesk.controller.command.scheduler.quartz.notif.exec.InitializeSchedulerExecNotificationsTab.notificationRulesIndexDisabledEmptyText' ) );
        }
      }
      else
      {
        filterQueryText.setEmptyText(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.notif.exec.InitializeSchedulerExecNotificationsTab.indexAgentNotReadyEmptyText' ) );
      }

      execNotificationRulesGrid.getStore().on( 'beforeload', function ( store, operation )
      {
        execNotificationRulesGrid.down( '#refreshButton' ).disable();
        execNotificationRulesGrid.down( '#addButton' ).disable();
        execNotificationRulesGrid.down( '#deleteButton' ).disable();
        execNotificationRulesGrid.down( '#editButton' ).disable();
        execNotificationRulesGrid.down( '#cloneButton' ).disable();
        execNotificationRulesGrid.down( '#viewAuditTrailButton' ).disable();
        filterQueryText.disable();

        /*
         * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
         * Therefore we need to force-fire this event by clearing the selection.
         */
        execNotificationRulesGrid.getSelectionModel().deselectAll();
      } );

      execNotificationRulesGrid.getStore().on( 'load', function ( store, records, successful, operation )
      {
        execNotificationRulesGrid.down( '#refreshButton' ).enable();

        // enable only if the notification rules index is enabled in the agent
        if ( notificationRulesIndexEnabled )
        {
          filterQueryText.enable();
        }

        if ( successful )
        {
          execNotificationRulesGrid.down( '#addButton' ).enable();
        }
        else
        {
          store.removeAll();

          var response = operation.getResponse();
          if ( response && response.responseText )
          {
            // application errors
            QDesk.util.UxUtils.showJsonResponseErrors( Ext.JSON.decode( response.responseText ) );
          }
          else
          {
            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get(
                    'QDesk.controller.command.scheduler.quartz.notif.exec.InitializeSchedulerExecNotificationsTab.errorLoadingNotifRulesMsg' ) );
          }
        }
      } );

      execNotificationRulesGrid.on( 'addRule',
          function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.AddExecNotificationRule', {
              connectionId : me.connectionId,
              /*
               * Upon success refresh the notifications tab.
               */
              success : function ()
              {
                Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab',
                    {
                      connectionId : me.connectionId
                    } ).execute();
              }
            } ).execute();
          } );

      execNotificationRulesGrid.on( 'enableRule',
          function ( execNotifRule )
          {
            me._enableExecNotificationRule( execNotifRule );
          } );

      execNotificationRulesGrid.on( 'disableRule',
          function ( execNotifRule )
          {
            me._disableExecNotificationRule( execNotifRule );
          } );

      execNotificationRulesGrid.on( 'deleteRule',
          function ( execNotifRule )
          {
            me._deleteNotificationRule( execNotifRule );
          } );

      execNotificationRulesGrid.on( 'editRule',
          function ( execNotifRule )
          {
            me._editExecNotificationRule( execNotifRule );
          } );

      execNotificationRulesGrid.on( 'cloneRule',
          function ( execNotifRule )
          {
            me._cloneExecNotificationRule( execNotifRule );
          } );

      execNotificationRulesGrid.on( 'viewAuditTrail',
          function ( execNotifRule )
          {
            me._viewExecNotificationRuleAuditTrail( execNotifRule );
          } );

      // opens a window with messages enqueued by the notification rule
      execNotificationRulesGrid.down( '#messagesColumn' ).on( 'requestViewMessages',
          function ( execNotificationRuleId )
          {
            Ext.create( 'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator', {
              connectionId : me.connectionId,
              creatorId : execNotificationRuleId,
              creatorType : QDesk.model.message.MessageCreatorType.QUARTZ_EXEC_NOTIF_RULE
            } ).execute();
          } );
    }
  },


  /**
   * Invoked when a notification rule record in the SchedulerExecNotificationRulesGrid in the SchedulerNotificationsTab
   * is selected.
   * @param notificationRule a QDesk.model.scheduler.quartz.QuartzExecNotificationRule instance representing the selected
   * notification rule.
   * @param schedulerNotificationsTab the scheduler Notifications tab.
   */
  _onNotificationRuleSelected : function ( notificationRule, schedulerNotificationsTab )
  {
    var me = this;

    var notificationRulesGrid = schedulerNotificationsTab.down( '#execNotificationRulesGrid' );

    notificationRulesGrid.down( '#deleteButton' ).enable();
    notificationRulesGrid.down( '#editButton' ).enable();
    notificationRulesGrid.down( '#cloneButton' ).enable();
    notificationRulesGrid.down( '#viewAuditTrailButton' ).enable();
  },


  /**
   * Invoked when a notification rule record in the SchedulerExecNotificationRulesGrid in the SchedulerNotificationsTab
   * is unselected.
   * @param schedulerNotificationsTab the scheduler Notifications tab.
   */
  _onNotificationRuleUnselected : function ( schedulerNotificationsTab )
  {
    var me = this;

    var notificationRulesGrid = schedulerNotificationsTab.down( '#execNotificationRulesGrid' );

    notificationRulesGrid.down( '#deleteButton' ).disable();
    notificationRulesGrid.down( '#editButton' ).disable();
    notificationRulesGrid.down( '#cloneButton' ).disable();
    notificationRulesGrid.down( '#viewAuditTrailButton' ).disable();
  },


  /**
   * Invoked to refresh the data in the scheduler execution notification rules grid.
   * @param connectionId a connection ID.
   */
  _refreshNotificationRulesGrid : function ( connectionId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationRulesGrid', {
      connectionId : connectionId
    } ).execute();
  },


  _enableExecNotificationRule : function ( execNotifRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.EnableExecNotificationRule', {
      connectionId : me.connectionId,
      notificationRule : execNotifRule,
      /*
       * Upon success refresh the notifications tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _disableExecNotificationRule : function ( execNotifRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.DisableExecNotificationRule', {
      connectionId : me.connectionId,
      notificationRule : execNotifRule,
      /*
       * Upon success refresh the notifications tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _deleteNotificationRule : function ( execNotifRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.DeleteExecNotificationRule', {
      connectionId : me.connectionId,
      notificationRule : execNotifRule,
      /*
       * Upon success refresh the notifications tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _editExecNotificationRule : function ( execNotifRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule', {
      connectionId : me.connectionId,
      notificationRuleId : execNotifRule.get( 'id' ),
      /*
       * Upon success refresh the notifications tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _cloneExecNotificationRule : function ( execNotifRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule', {
      connectionId : me.connectionId,
      notificationRuleId : execNotifRule.get( 'id' ),
      /*
       * Upon success refresh the notifications tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _viewExecNotificationRuleAuditTrail : function ( execNotifRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.ViewExecNotificationRuleAuditTrail', {
      connectionId : me.connectionId,
      notificationRuleId : execNotifRule.get( 'id' )
    } ).execute();
  }
} );
