/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Edits an existing trigger.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.triggers.EditTrigger', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.util.CommonUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.StoreUtils',
    'QDesk.util.UxUtils',

    'QDesk.util.constant.Const',
    'QDesk.util.constant.EventName',

    'QDesk.util.quartz.QuartzTriggerUtils',

    'QDesk.view.work.scheduler.quartz.triggers.EditTriggerFormMode',
    'QDesk.view.work.scheduler.quartz.triggers.EditTriggerWindow'
  ],

  /*
   * Required ID of the scheduler connection.
   */
  connectionId : undefined,

  /*
   * Required QDesk.model.scheduler.quartz.QuartzTrigger instance representing the trigger to edit.
   */
  trigger : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.trigger )
      Ext.Error.raise( 'Required config parameter missing: trigger' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.EditTrigger.maskLoadMsg' ) );

    var jvmAgentInfo = QDesk.controller.command.connection.ConnectionUtils.getJvmAgentInfo( me.connectionId );
    var schedulerInfo = QDesk.controller.command.connection.ConnectionUtils.getSchedulerInfo( me.connectionId );

    var window = Ext.widget( 'QuartzEditTriggerWindow', {
      qdesk : {
        connectionId : me.connectionId,
        mode : QDesk.view.work.scheduler.quartz.triggers.EditTriggerFormMode.EDIT,
        triggerType : me.trigger.get( 'type' ),
        triggerJobGroupName : me.trigger.get( 'jobGroup' ).name,
        triggerJobName : me.trigger.get( 'jobName' ),
        jvmAgentInfo : jvmAgentInfo,
        schedulerInfo : schedulerInfo
      }
    } );

    var formPanel = window.down( '#form' );

    window.on( 'afterrender', function ()
    {
      var nameField = formPanel.down( '#name' );
      QDesk.util.UxUtils.focus( nameField, false );
    } );

    // Save/Update button
    formPanel.down( '#actionButton' ).on( 'click', function ()
    {
      me._submitForm( window );
    } );

    var formStores = formPanel.getFormStores();

    /* Load the form data */
    QDesk.util.StoreUtils.onAllStoresLoad( formStores,
        function ( successful, firstFailedOperation )
        {
          // load trigger data into the form (the loadRecord method is overridden in trigger forms
          if ( successful )
          {
            formPanel.loadRecord( me.trigger );
          }

          me.unmask();

          if ( successful )
          {
            window.show();
          }
          else
          {
            var response = firstFailedOperation.getResponse();
            if ( response && response.responseText )
            {
              // application errors
              QDesk.util.UxUtils.showJsonResponseErrors( Ext.JSON.decode( response.responseText ) );
            }
            else
            {
              // error loading stores
              QDesk.util.UxUtils.showErrorDialog(
                  QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.EditTrigger.errorLoadMsg' ) );
            }
          }
        } );
    QDesk.util.StoreUtils.loadStores( formStores );
  },


  /**
   * Submits the form.
   * @param window the edit connection window.
   */
  _submitForm : function ( window )
  {
    var me = this;

    var formPanel = window.down( '#form' );

    //var form = formPanel.getForm();
    //var fieldValues = form.getFieldValues();
    //var jobDataMap = formPanel.getJobDataMap();
    //
    //var triggerGroupName = fieldValues.group;
    //var triggerName = fieldValues.name;
    //
    //var trigger = QDesk.util.quartz.QuartzTriggerUtils.createTrigger( fieldValues, jobDataMap );

    var form = formPanel.getForm();
    var trigger = formPanel.getTriggerRecord();
    var triggerGroupName = trigger.get( 'group' ).name;
    var triggerName = trigger.get( 'name' );

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.quartz.triggerActionUpdate,
      params : {
        connectionId : me.connectionId,
        trigger : QDesk.util.CommonUtils.encode( trigger )
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.EditTrigger.waitMsg',
          Ext.String.htmlEncode( triggerGroupName ), Ext.String.htmlEncode( triggerName ) ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          window.close();

          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.EditTrigger.successMsg',
                  Ext.String.htmlEncode( triggerGroupName ), Ext.String.htmlEncode( triggerName ) ),
              function ()
              {
                me.application.fireEvent( QDesk.util.constant.EventName.QUARTZ.TRIGGER_UPDATED, {
                  connectionId : me.connectionId,
                  trigger : trigger
                } );

                // invoke the success callback handler
                if ( me.success && Ext.isFunction( me.success ) )
                {
                  me.success.apply( me );
                }
              } );
        }
        else
        {
          QDesk.util.UxUtils.showJsonResponseErrors( json,
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action,
            function ()
            {
              // invoke the failure callback handler
              if ( me.failure && Ext.isFunction( me.failure ) )
              {
                me.failure.apply( me );
              }
            } );
      }
    } );
  }
} );
