/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Logs in the specified user.
 */
Ext.define( 'QDesk.controller.command.security.Login', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.CurrentUser',
    'QDesk.Msg',
    'QDesk.model.common.Result',
    'QDesk.model.security.LoginError',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.EventName'
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.username )
      Ext.Error.raise( 'Required config parameter missing: username' );

    if ( !config.password )
      Ext.Error.raise( 'Required config parameter missing: password' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.security.Login.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.securityLoginAction,
      params : {
        username : me.username,
        password : me.password
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            var result = json.result;
            if ( result.result === QDesk.model.common.Result.SUCCESS )
            {
              // update the current user's data
              QDesk.CurrentUser.update(
                  result.sessionToken,
                  {
                    authenticated : true,
                    userProfile : result.userProfile
                  },
                  false );

              me.application.fireEvent( QDesk.util.constant.EventName.LOGIN_SUCCESS, {
                sessionToken : result.sessionToken,
                userProfile : result.userProfile
              } );
            }
            else
            {
              // based on result.error code show the relevant error message.
              if ( result.error )
              {
                switch ( result.error )
                {
                  case QDesk.model.security.LoginError.INVALID_USERNAME_OR_PASSWORD:
                    QDesk.util.UxUtils.showWarningDialog(
                        QDesk.Msg.get( 'QDesk.controller.command.security.Login.errorInvalidUsernameOrPassword' ) );
                    break;

                  case QDesk.model.security.LoginError.ALREADY_AUTHENTICATED_IN_THIS_SESSION:
                    QDesk.util.UxUtils.showWarningDialog( QDesk.Msg.get(
                        'QDesk.controller.command.security.Login.errorAlreadyAuthenticatedInThisSession' ) );
                    break;

                  case QDesk.model.security.LoginError.ALREADY_AUTHENTICATED_IN_ANOTHER_SESSION:
                    QDesk.util.UxUtils.showWarningDialog( QDesk.Msg.get(
                        'QDesk.controller.command.security.Login.errorAlreadyAuthenticatedInAnotherSession' ) );

                    // TODO: ask the user to terminate the other session(s)

                    break;

                  case QDesk.model.security.LoginError.ACCOUNT_LOCKED_MANUALLY:
                    QDesk.util.UxUtils.showWarningDialog(
                        QDesk.Msg.get( 'QDesk.controller.command.security.Login.errorAccountLockedManually' ) );
                    break;

                  case QDesk.model.security.LoginError.ACCOUNT_LOCKED_AUTO_TOO_MANY_LOGIN_ATTEMPTS:
                    // result.resultDetail contains the datetime in the ISO8601 format the account has been locked till
                    QDesk.util.UxUtils.showWarningDialog( QDesk.Msg.get(
                        'QDesk.controller.command.security.Login.errorAccountLockedAutoTooManyLoginAttempts',
                        QDesk.util.FormatUtils.parseAndFormatTimestamp( result.resultDetail ) ) );
                    break;

                  case QDesk.model.security.LoginError.ACCOUNT_DELETED:
                    QDesk.util.UxUtils.showWarningDialog(
                        QDesk.Msg.get( 'QDesk.controller.command.security.Login.errorAccountDeleted' ) );
                    break;

                  default:
                    QDesk.util.UxUtils.showErrorDialog(
                        QDesk.Msg.get( 'QDesk.controller.command.security.Login.errorUnexpectedErrorCode',
                            json.error ) );
                    break;
                }
              }

              me.application.fireEvent( QDesk.util.constant.EventName.LOGIN_FAILED, {
                sessionToken : result.sessionToken,
                username : me.username
              } );
            }
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog( QDesk.Msg.get( 'QDesk.controller.command.security.Login.errorMsg' ) );
        }
      }
    } );
  }
} );
