/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


Ext.define( 'QDesk.util.FormatUtils', {
  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Log',
    'QDesk.Msg'
  ],

  singleton : true,

  /**
   * Number of milliseconds in a second.
   */
  MILLS_IN_SECOND : 1000,

  /**
   * Number of seconds in a minute.
   */
  SECONDS_IN_MINUTE : 60,

  /**
   * Number of minutes in an hour.
   */
  MINUTES_IN_HOUR : 60,

  /**
   * Number of hours in a day.
   */
  HOURS_IN_DAY : 24,

  /**
   * Number of milliseconds in a minute.
   */
  MILLS_IN_MINUTE : 60000,

  /**
   * Number of milliseconds in an hour.
   */
  MILLS_IN_HOUR : 3600000,

  /**
   * Number of milliseconds in a day.
   */
  MILLS_IN_DAY : 86400000,

  /**
   * Number of seconds in a hour.
   */
  SECONDS_IN_HOUR : 3600,

  /**
   * Number of seconds in a day.
   */
  SECONDS_IN_DAY : 86400,


  /**
   * Pretty-prints the specified object and returns the result.
   * @param obj an object.
   * @returns {*}
   */
  prettyPrint : function ( obj )
  {
    if ( JSON && JSON.stringify )
    {
      return JSON.stringify( obj, null, 2 );
    }
    else
    {
      return obj;
    }
  },


  /**
   * Formats the specified number in hex.
   * @param value a number to format.
   */
  formatNumberInHex : function ( value )
  {
    if ( value == null )
      return null;

    if ( Ext.isNumber( value ) )
    {
      return '0x' + value.toString( 16 );
    }

    return null;
  },


  /**
   * Formats the specified boolean value as Yes or No.
   * @param value a boolean value to format.
   */
  formatBoolean : function ( value )
  {
    if ( value == null )
      return null;

    return value ? QDesk.Msg.get( 'QDesk.yes' ) : QDesk.Msg.get( 'QDesk.no' );
  },


  /**
   * Parses the specified timestamp value using the {@link QDesk.util.constant.DateFormat#timestampFormatIso8601}
   * timestamp format and formats it using the configured timestamp format.
   * @param value a timestamp value as a string in the ISO8601 format.
   * @return the formatted timestamp value.
   */
  parseAndFormatTimestamp : function ( value )
  {
    var timestamp = QDesk.util.FormatUtils.parseTimestamp( value );
    return QDesk.util.FormatUtils.formatTimestamp( timestamp );
  },


  /**
   * Parses the specified timestamp value using the ({@link QDesk.util.constant.DateFormat#timestampFormatIso8601})
   * format and returns a Date instance.
   * @param value a timestamp value as a ISO8601 string.
   * @return the parsed Date value.
   */
  parseTimestamp : function ( value )
  {
    if ( value == null )
      return null;

    return Ext.Date.parse( value, QDesk.util.constant.DateFormat.timestampFormatIso8601 );
  },


  /**
   * Formats the specified timestamp (Date) value using the configured
   * timestamp format and returns the result.
   * @param value a timestamp value as a Date.
   * @return the formatted timestamp value as a String.
   */
  formatTimestamp : function ( value )
  {
    if ( value == null )
      return null;

    //return Ext.Date.format( value, QDesk.util.constant.DateFormat.timestampFormatLong );
    return Ext.Date.format( value, QDesk.ApplicationConfig.get( 'timestampFormat' ) );
  },


  /**
   * Parses the specified time value using the configured time format
   * ({@link QDesk.util.constant.DateFormat#dayTimeFormat}) and
   * returns the resulting Date.
   * @param value a time value as a string.
   * @return the parsed time value as a Date.
   */
  parseDayTime : function ( value )
  {
    if ( value == null )
      return null;

    return Ext.Date.parse( value, QDesk.util.constant.DateFormat.dayTimeFormat );
  },


  /**
   * Formats the specified time (Date) value using the
   * {@link QDesk.util.constant.DateFormat#dayTimeFormat}) time format
   * returns the result.
   * @param value a time value as a Date.
   * @return the formatted time value as a String.
   */
  formatDayTime : function ( value )
  {
    if ( value == null )
      return null;

    return Ext.Date.format( value, QDesk.util.constant.DateFormat.dayTimeFormat );
  },


  /**
   * Parses the specified time value using the configured time format
   * ({@link QDesk.util.constant.DateFormat#timeFormatMedium}) and
   * returns the resulting Date.
   * @param value a time value as a string.
   * @return the parsed time value as a Date.
   */
  parseTimeMedium : function ( value )
  {
    if ( value == null )
      return null;

    return Ext.Date.parse( value, QDesk.util.constant.DateFormat.timeFormatMedium );
  },


  /**
   * Formats the specified time (Date) value using the
   * {@link QDesk.util.constant.DateFormat#timeFormatMedium}) time format
   * returns the result.
   * @param value a time value as a Date.
   * @return the formatted time value as a String.
   */
  formatTimeMedium : function ( value )
  {
    if ( value == null )
      return null;

    return Ext.Date.format( value, QDesk.util.constant.DateFormat.timeFormatMedium );
  },


  /**
   * Formats the specified duration value in milliseconds.
   * @param duration the duration in milliseconds.
   */
  formatDuration : function ( duration )
  {
    var dur = duration;

    var days = Math.floor( dur / QDesk.util.FormatUtils.MILLS_IN_DAY );
    dur -= days * QDesk.util.FormatUtils.MILLS_IN_DAY;

    var hours = Math.floor( dur / QDesk.util.FormatUtils.MILLS_IN_HOUR );
    dur -= hours * QDesk.util.FormatUtils.MILLS_IN_HOUR;

    var minutes = Math.floor( dur / QDesk.util.FormatUtils.MILLS_IN_MINUTE );
    dur -= minutes * QDesk.util.FormatUtils.MILLS_IN_MINUTE;

    var seconds = Math.floor( dur / QDesk.util.FormatUtils.MILLS_IN_SECOND );
    dur -= seconds * QDesk.util.FormatUtils.MILLS_IN_SECOND;

    var millis = dur;

    var durationArgs = [];

    durationArgs[ 0 ] = days;

    durationArgs[ 1 ] =
        hours < 10 ? '0' + hours : hours.toString();

    durationArgs[ 2 ] =
        minutes < 10 ? '0' + minutes : minutes.toString();

    durationArgs[ 3 ] =
        seconds < 10 ? '0' + seconds : seconds.toString();

    durationArgs[ 4 ] =
        millis < 100 ? (millis < 10 ? '00' + millis : '0' + millis) : millis.toString();

    var result = '';

    if ( days > 0 )
    {
      result += days + QDesk.Msg.get( 'QDesk.util.FormatUtils.durationDaySuffix' ) + ' ';
    }

    //{hours}:{minutes}:{seconds}.{milliseconds}
    result += durationArgs[ 1 ] + ':' + durationArgs[ 2 ] + ':' + durationArgs[ 3 ] + '.' + durationArgs[ 4 ];

    return result;
  },


  /**
   * Formats the specified product version and returns the result.
   * @param productVersion the product version.
   */
  formatProductVersion : function ( productVersion )
  {
    var result = productVersion.major;
    result += '.';
    result += productVersion.minor;
    result += '.';
    result += productVersion.maintenance;

    if ( productVersion.qualifier )
    {
      result += '-';
      result += productVersion.qualifier;
    }

    return result;
  },


  /**
   * Formats the specified array of Java stack trace elements to a String.
   * @param stackTraceElements an array of Java stack trace elements.
   * @see com.quartzdesk.domain.model.common.jvm.StackTraceElement
   */
  formatStackTrace : function ( stackTraceElements )
  {
    if ( Ext.isEmpty( stackTraceElements ) )
      return null;

    var result = '<div class="stackTrace">';

    Ext.each( stackTraceElements,
        function ( stackTraceElement )
        {
          result += '<div class="element">';

          result += '<span class="className">' + stackTraceElement.className + '</span>';
          result += '.';
          result += '<span class="methodName">' + stackTraceElement.methodName + '</span>';

          if ( stackTraceElement.nativeMethod )
          {
            result += '(<span class="fileName">Native Method</span>)';
          }
          else
          {
            if ( stackTraceElement.fileName && stackTraceElement.lineNumber )
            {
              result += '(<span class="fileName">' + stackTraceElement.fileName + '</span>:' +
                  '<span class="lineNumber">' + stackTraceElement.lineNumber + '</span>)';
            }
            else
            {
              result += '(<span class="fileName">Unknown Source</span>)';
            }
          }

          result += '</div>';
        } );

    result += "</div>";

    return result;
  },


  /**
   * Formats the specified product name and product version and returns
   * the result.
   * @param productName the product name.
   * @param productVersion the product version.
   */
  formatProductNameAndVersion : function ( productName, productVersion )
  {
    var result = productName;
    if ( productVersion )
    {
      result += ' v';
      result += productVersion.major;
      result += '.';
      result += productVersion.minor;
      result += '.';
      result += productVersion.maintenance;

      if ( productVersion.qualifier )
      {
        result += '-';
        result += productVersion.qualifier;
      }

      return result;
    }
  },


  /**
   * Formats the specified list of permissions as a HTML text and returns the result.
   * @param permissions the list of permissions.
   * @return the HTML-formatted list of permissions.
   */
  htmlFormatPermissions : function ( permissions )
  {
    var result = '[';
    Ext.each( permissions, function ( permission, index )
    {
      result += '<br/>';
      result += '<span style="padding-left: 10px">' + permission + '</span>';

      if ( index === permissions.length - 1 )
        result += '<br/>';
    } );

    result += ']';
    return result;
  },


  /**
   * Strips all HTML tags from the specified text and returns the result.
   * @param text an input text.
   * @returns the result.
   */
  stripHtmlTags : function ( text )
  {
    if ( text )
    {
      return text.replace( /<\/?[^>]+(>|$)/g, "" );
    }
    else
    {
      return text;
    }
  }
} );
