/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


/**
 * Various message channel profile related utility methods.
 */
Ext.define( 'QDesk.util.MessageChannelProfileUtils', {
  singleton : true,

  requires : [
    'QDesk.model.message.channel.MessageChannel',
    'QDesk.view.work.message.channel.collaboration.hipchat.HipChatProfilePanel',
    'QDesk.view.work.message.channel.collaboration.slack.SlackProfilePanel',

    'QDesk.view.work.message.channel.email.EmailProfilePanel',

    'QDesk.view.work.message.channel.im.aim.AimProfilePanel',
    'QDesk.view.work.message.channel.im.gtalk.GTalkProfilePanel',
    'QDesk.view.work.message.channel.im.icq.IcqProfilePanel',
    'QDesk.view.work.message.channel.im.xmpp.XmppProfilePanel',
    'QDesk.view.work.message.channel.im.yahoo.YahooProfilePanel',

    'QDesk.view.work.message.channel.sms.clickatell.central.ClickatellProfilePanel',
    'QDesk.view.work.message.channel.sms.clickatell.platform.ClickatellProfilePanel',
    'QDesk.view.work.message.channel.sms.clockwork.ClockworkProfilePanel',
    'QDesk.view.work.message.channel.sms.nexmo.NexmoProfilePanel',
    'QDesk.view.work.message.channel.sms.smsglobal.SmsGlobalProfilePanel',
    'QDesk.view.work.message.channel.sms.twilio.TwilioProfilePanel',

    'QDesk.view.work.message.channel.snmp.SnmpV1TrapProfilePanel',
    'QDesk.view.work.message.channel.snmp.SnmpV2cTrapProfilePanel',
    'QDesk.view.work.message.channel.snmp.SnmpV3TrapProfilePanel',

    'QDesk.view.work.message.channel.web_service.WebServiceProfilePanel'
  ],


  /**
   * Returns the message channel profiles window.
   * @return the window.
   */
  getProfilesWindow : function ()
  {
    var window = Ext.ComponentQuery.query( 'MessageChannelProfilesWindow' )[ 0 ];

    if ( !window )
      Ext.Error.raise( 'Message channel profiles window not found.' );

    return window;
  },


  /**
   * Returns the message channel profiles tree panel.
   * @return the tree panel.
   */
  getProfilesTreePanel : function ()
  {
    var window = QDesk.util.MessageChannelProfileUtils.getProfilesWindow();
    return window.down( '#messageChannelProfilesTreePanel' );
  },


  /**
   * Returns the message channel profiles edit panel.
   * @return the edit panel.
   */
  getEditPanel : function ()
  {
    var window = QDesk.util.MessageChannelProfileUtils.getProfilesWindow();
    return window.down( '#editPanel' );
  },


  /**
   * Adds the specified form panel to the edit panel and makes the panel active (edit panel uses card layout).
   * @param panel a panel to add.
   */
  addAndShowPanel : function ( panel )
  {
    var editPanel = QDesk.util.MessageChannelProfileUtils.getEditPanel();
    editPanel.add( panel );
    editPanel.getLayout().setActiveItem( panel );
  },


  /**
   * Returns the node ID of the specified profile.
   * @param profileId a profile ID.
   * @return the node ID of the specified profile.
   */
  getProfileNodeId : function ( profileId )
  {
    return 'profile-' + profileId;
  },


  /**
   * Returns the name of the profile with the specified ID.
   * @param profileId a profile ID.
   */
  getProfileName : function ( profileId )
  {
    var treePanel = QDesk.util.MessageChannelProfileUtils.getProfilesTreePanel();
    var store = treePanel.getStore();

    var nodeId = QDesk.util.MessageChannelProfileUtils.getProfileNodeId( profileId );
    var profileNode = store.getNodeById( nodeId );
    return profileNode.get( 'name' );
  },


  /**
   * Returns the ID of the parent folder of the specified profile, or null if not found.
   * @param profileId a profile ID.
   * @return the parent folder ID, or null.
   */
  getProfileParentFolderId : function ( profileId )
  {
    var treePanel = QDesk.util.MessageChannelProfileUtils.getProfilesTreePanel();
    var store = treePanel.getStore();

    var nodeId = QDesk.util.MessageChannelProfileUtils.getProfileNodeId( profileId );
    var profileNode = store.getNodeById( nodeId );
    return profileNode ? profileNode.parentNode.get( 'folderId' ) : null;
  },


  /**
   * Returns the node ID of the specified folder.
   * @param folderId a folder ID.
   * @return the node ID of the specified folder.
   */
  getFolderNodeId : function ( folderId )
  {
    return folderId == QDesk.util.constant.Const.MESSAGE_CHANNEL_PROFILES_ROOT_FOLDER_ID ?
        QDesk.util.constant.Const.MESSAGE_CHANNEL_PROFILES_ROOT_ID : 'folder-' + folderId;
  },


  /**
   * Returns the name of the folder with the specified ID.
   * @param folderId a folder ID.
   */
  getFolderName : function ( folderId )
  {
    var treePanel = QDesk.util.MessageChannelProfileUtils.getProfilesTreePanel();
    var store = treePanel.getStore();

    var nodeId = QDesk.util.MessageChannelProfileUtils.getFolderNodeId( folderId );
    var folderNode = store.getNodeById( nodeId );
    return folderNode.get( 'name' );
  },


  /**
   * Returns the ID of the parent folder of the specified folder, or null if not found.
   * @param folderId a folder ID.
   * @return the parent folder ID, or null.
   */
  getFolderParentFolderId : function ( folderId )
  {
    var treePanel = QDesk.util.MessageChannelProfileUtils.getProfilesTreePanel();
    var store = treePanel.getStore();

    var nodeId = QDesk.util.MessageChannelProfileUtils.getFolderNodeId( folderId );
    var folderNode = store.getNodeById( nodeId );
    return folderNode ? folderNode.parentNode.get( 'folderId' ) : null;
  },


  /**
   * Returns the {@link QDesk.model.message.channel.MessageChannel} for the specified profile ID.
   * @param profileId a profile ID.
   * @return the {@link QDesk.model.message.channel.MessageChannel} of the specified profile.
   */
  getProfileMessageChannel : function ( profileId )
  {
    var treePanel = QDesk.util.MessageChannelProfileUtils.getProfilesTreePanel();
    var store = treePanel.getStore();

    var nodeId = QDesk.util.MessageChannelProfileUtils.getProfileNodeId( profileId );
    var profileNode = store.getNodeById( nodeId );
    return profileNode.get( 'messageChannel' );
  },


  cleanupEditPanel : function ()
  {
    var editPanel = QDesk.util.MessageChannelProfileUtils.getEditPanel();

    // remove all panels (profile editors) from the edit panel (right panel), but the infoPanel
    Ext.each( editPanel.getLayout().getLayoutItems(), function ( panel )
    {
      if ( panel.getItemId() === 'infoPanel' )
      {
        // edit panel uses card layout, so we activate the info panel
        editPanel.getLayout().setActiveItem( panel );
      }
      else
      {
        editPanel.remove( panel );
      }
    } );
  },


  /**
   * Creates and returns an empty profile panel for the specified message channel.
   * @param messageChannel a message channel.
   * @return the created profile panel.
   */
  createEmptyProfilePanel : function ( messageChannel )
  {
    switch ( messageChannel )
    {
        /*
         * Collaboration message channels.
         */
      case QDesk.model.message.channel.MessageChannel.COLLABORATION_HIPCHAT:
        return Ext.widget( 'CollaborationHipChatProfilePanel' );

      case QDesk.model.message.channel.MessageChannel.COLLABORATION_SLACK:
        return Ext.widget( 'CollaborationSlackProfilePanel' );

        /*
         * Email message channel.
         */
      case QDesk.model.message.channel.MessageChannel.EMAIL:
        return Ext.widget( 'EmailProfilePanel' );

        /*
         * IM message channels.
         */
      case QDesk.model.message.channel.MessageChannel.IM_AIM:
        return Ext.widget( 'ImAimProfilePanel' );

      case QDesk.model.message.channel.MessageChannel.IM_GTALK:
        return Ext.widget( 'ImGTalkProfilePanel' );

      case QDesk.model.message.channel.MessageChannel.IM_ICQ:
        return Ext.widget( 'ImIcqProfilePanel' );

      case QDesk.model.message.channel.MessageChannel.IM_XMPP:
        return Ext.widget( 'ImXmppProfilePanel' );

      case QDesk.model.message.channel.MessageChannel.IM_YAHOO:
        return Ext.widget( 'ImYahooProfilePanel' );

        /*
         * SMS message channels.
         */
      case QDesk.model.message.channel.MessageChannel.SMS_CLICKATELL_CENTRAL:
        return Ext.widget( 'SmsClickatellCentralProfilePanel' );

      case QDesk.model.message.channel.MessageChannel.SMS_CLICKATELL_PLATFORM:
        return Ext.widget( 'SmsClickatellPlatformProfilePanel' );

      case QDesk.model.message.channel.MessageChannel.SMS_CLOCKWORK:
        return Ext.widget( 'SmsClockworkProfilePanel' );

      case QDesk.model.message.channel.MessageChannel.SMS_NEXMO:
        return Ext.widget( 'SmsNexmoProfilePanel' );

      case QDesk.model.message.channel.MessageChannel.SMS_SMSGLOBAL:
        return Ext.widget( 'SmsSmsGlobalProfilePanel' );

      case QDesk.model.message.channel.MessageChannel.SMS_TWILIO:
        return Ext.widget( 'SmsTwilioProfilePanel' );

        /*
         * SNMP message channels.
         */
      case QDesk.model.message.channel.MessageChannel.SNMPV1_TRAP:
        return Ext.widget( 'SnmpV1TrapProfilePanel' );

      case QDesk.model.message.channel.MessageChannel.SNMPV2C_TRAP:
        return Ext.widget( 'SnmpV2cTrapProfilePanel' );

      case QDesk.model.message.channel.MessageChannel.SNMPV3_TRAP:
        return Ext.widget( 'SnmpV3TrapProfilePanel' );

        /*
         * Web Service message channels.
         */
      case QDesk.model.message.channel.MessageChannel.WEB_SERVICE:
        return Ext.widget( 'WebServiceProfilePanel' );

      default:
        Ext.Error.raise( 'Unsupported message channel: ' + messageChannel );
    }
  },


  /**
   * Returns true if the specified message channel supports recipients.
   * @param messageChannel a message channel.
   * @returns {boolean} true if the message channel supports recipients, false otherwise.
   */
  messageChannelSupportsRecipients : function ( messageChannel )
  {
    return messageChannel === QDesk.model.message.channel.MessageChannel.EMAIL
        /* SMS channels */
        || messageChannel === QDesk.model.message.channel.MessageChannel.SMS_CLICKATELL_CENTRAL
        || messageChannel === QDesk.model.message.channel.MessageChannel.SMS_CLICKATELL_PLATFORM
        || messageChannel === QDesk.model.message.channel.MessageChannel.SMS_CLOCKWORK
        || messageChannel === QDesk.model.message.channel.MessageChannel.SMS_NEXMO
        || messageChannel === QDesk.model.message.channel.MessageChannel.SMS_SMSGLOBAL
        || messageChannel === QDesk.model.message.channel.MessageChannel.SMS_TWILIO

        /* AIM channels */
        || messageChannel === QDesk.model.message.channel.MessageChannel.IM_AIM
        || messageChannel === QDesk.model.message.channel.MessageChannel.IM_GTALK
        || messageChannel === QDesk.model.message.channel.MessageChannel.IM_ICQ
        || messageChannel === QDesk.model.message.channel.MessageChannel.IM_XMPP
        || messageChannel === QDesk.model.message.channel.MessageChannel.IM_YAHOO;
  },


  /**
   * Returns true if the specified message channel supports a message subject.
   * @param messageChannel a message channel.
   * @returns {boolean} true if the message channel supports a message subject, false otherwise.
   */
  messageChannelSupportsSubject : function ( messageChannel )
  {
    return messageChannel === QDesk.model.message.channel.MessageChannel.EMAIL
        || messageChannel === QDesk.model.message.channel.MessageChannel.SNMPV1_TRAP
        || messageChannel === QDesk.model.message.channel.MessageChannel.SNMPV2C_TRAP
        || messageChannel === QDesk.model.message.channel.MessageChannel.SNMPV3_TRAP
        || messageChannel === QDesk.model.message.channel.MessageChannel.WEB_SERVICE;
  },


  /**
   * Returns true if the specified message channel supports attachments.
   * @param messageChannel a message channel.
   * @returns {boolean} true if the message channel supports attachments, false otherwise.
   */
  messageChannelSupportsAttachments : function ( messageChannel )
  {
    return messageChannel === QDesk.model.message.channel.MessageChannel.COLLABORATION_HIPCHAT
        || messageChannel === QDesk.model.message.channel.MessageChannel.COLLABORATION_SLACK
        || messageChannel === QDesk.model.message.channel.MessageChannel.EMAIL
        || messageChannel === QDesk.model.message.channel.MessageChannel.WEB_SERVICE;
  }
} );
