/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Base class for a scheduler view tabs. This class provides access to the
 * tab initialization flag and scheduler description object in the scheduler view.
 */
Ext.define( 'QDesk.view.work.scheduler.BaseSchedulerSubTab', {
  extend : 'QDesk.view.widget.panel.XPanel',

  requires : [
    'QDesk.util.license.LicensedProduct'
  ],

  bodyCls : 'baseSchedulerSubTabBody',

  closable : false,

  scrollable : true,

  // /**
  //  * Returns the value of the QDesk boolean flag indicating whether tab has been initialized.
  //  * @return the QDesk initialization flag.
  //  */
  // getInitialized : function ()
  // {
  //   return this.qdesk.initialized;
  // },
  //
  //
  // /**
  //  * Sets the value of the QDesk boolean flag indicating whether tab has been initialized.
  //  * @param initialized the QDesk initialization flag.
  //  */
  // setInitialized : function ( initialized )
  // {
  //   this.qdesk.initialized = initialized;
  // },
  //

  /**
   * Returns the connection ID this tab has been created for.
   * @return the connection ID.
   */
  getConnectionId : function ()
  {
    var me = this;
    var schedulerView = me.up( 'panel' );
    // assert schedulerView != null
    return schedulerView.getConnectionId();
  },


  /**
   * Returns true if the QuartzDesk JVM Agent is detected (i.e. it is available and compatible) with the
   * QuartzDesk GUI, false otherwise.
   * @param jvmAgentInfo QDesk.JvmAgentInfoAccessor instance.
   * @returns true if the QuartzDesk JVM Agent is detected, false otherwise.
   */
  isJvmAgentDetected : function ( jvmAgentInfo )
  {
    return jvmAgentInfo.isDetected();
  },


  /**
   * Returns true if the QuartzDesk JVM Agent is ready (i.e. it is available and compatible) with the
   * QuartzDesk GUI, false otherwise.
   * @param jvmAgentInfo QDesk.JvmAgentInfoAccessor instance.
   * @returns true if the QuartzDesk JVM Agent is ready, false otherwise.
   */
  isJvmAgentReady : function ( jvmAgentInfo )
  {
    return jvmAgentInfo.isReady();
  },


  /**
   * Returns the QuartzDesk JVM Agent availability and compatibility warning message
   * that gets shown in the yellow box in the GUI if the agent is either not available,
   * or its version/edition is not compatible with the GUI.
   * @param schedulerInfo QDesk.SchedulerInfoAccessor instance.
   * @param jvmAgentInfo QDesk.JvmAgentInfoAccessor instance.
   * @returns the warning message.
   */
  getJvmAgentWarningMessage : function ( schedulerInfo, jvmAgentInfo )
  {
    if ( jvmAgentInfo.isReady() )
      return null;

    if ( jvmAgentInfo.isDetected() )
    {
      if ( !jvmAgentInfo.isCompatibleVersion() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentIncompatibleVersionMsg',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            QDesk.util.FormatUtils.formatProductVersion( jvmAgentInfo.getVersion() ),
            QDesk.util.FormatUtils.formatProductVersion( QDesk.util.license.LicensedProduct.productVersion ) );
      }
      else if ( !jvmAgentInfo.isCompatibleEdition() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentIncompatibleEditionMsg',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            jvmAgentInfo.getEdition(),
            QDesk.util.license.LicensedProduct.productEdition );
      }
      else
      {
        return null;
      }
    }
    else
    {
      return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentNotDetectedMsg',
          QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
    }
  },


  /**
   * Returns the QuartzDesk JVM Agent availability and compatibility warning tooltip that gets shown in GUI
   * if the agent is either not available, or its version/edition is not compatible with the GUI.
   * @param schedulerInfo scheduler info.
   * @param jvmAgentInfo the JVM Agent info.
   * @returns the warning message.
   */
  getJvmAgentWarningTooltip : function ( schedulerInfo, jvmAgentInfo )
  {
    if ( jvmAgentInfo.isReady() )
      return null;

    if ( jvmAgentInfo.isDetected() )
    {
      if ( !jvmAgentInfo.isCompatibleVersion() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentIncompatibleVersionTooltip',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            QDesk.util.FormatUtils.formatProductVersion( jvmAgentInfo.getVersion() ),
            QDesk.util.FormatUtils.formatProductVersion( QDesk.util.license.LicensedProduct.productVersion ) );
      }
      else if ( !jvmAgentInfo.isCompatibleEdition() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentIncompatibleEditionTooltip',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            jvmAgentInfo.getEdition(),
            QDesk.util.license.LicensedProduct.productEdition );
      }
      else
      {
        return null;
      }
    }
    else
    {
      return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentNotDetectedTooltip',
          QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
    }
  },


  /**
   * Returns true if the QuartzDesk Public API is ready (i.e. it is detected, loaded correctly and its version is
   * compatible with the agent), false otherwise.
   * @param schedulerInfo scheduler info.
   * @returns true if the QuartzDesk Public API is ready, false otherwise.
   */
  isPublicApiReady : function ( schedulerInfo )
  {
    return schedulerInfo.isPublicApiReady();
  },


  /**
   * Returns the QuartzDesk Public API availability and compatibility warning message that gets shown in the
   * yellow box in the GUI if the API is either not available, is not loaded correctly, or its version is not
   * compatible with the installed QuartzDesk JVM Agent.
   * @param schedulerInfo the scheduler info.
   * @param jvmAgentInfo the QuartzDesk JVM Agent info.
   * @returns the warning message.
   */
  getPublicApiWarningMessage : function ( schedulerInfo, jvmAgentInfo )
  {
    if ( schedulerInfo.isPublicApiReady() )
      return null;

    if ( schedulerInfo.isPublicApiDetected() )
    {
      if ( !schedulerInfo.isPublicApiLoadedCorrectly() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiNotLoadedCorrectlyMsg',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
      }
      else if ( !schedulerInfo.isPublicApiCompatibleVersion() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiIncompatibleVersionMsg',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            QDesk.util.FormatUtils.formatProductVersion( schedulerInfo.getPublicApiVersion() ),
            QDesk.util.FormatUtils.formatProductVersion( jvmAgentInfo.getVersion() ) );
      }
      else
      {
        return null;
      }
    }
    else
    {
      return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiNotDetectedMsg',
          QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
    }
  },


  /**
   * Returns the QuartzDesk Public API availability and compatibility warning tooltip that gets shown in the
   * the GUI if the API is either not detected, is not loaded correctly, or its version is not compatible
   * with the installed QuartzDesk JVM Agent.
   * @param schedulerInfo the scheduler info.
   * @param jvmAgentInfo the QuartzDesk JVM Agent info.
   * @returns the warning tooltip.
   */
  getPublicApiWarningTooltip : function ( schedulerInfo, jvmAgentInfo )
  {
    if ( schedulerInfo.isPublicApiReady() )
      return null;

    if ( schedulerInfo.isPublicApiDetected() )
    {
      if ( !schedulerInfo.isPublicApiLoadedCorrectly() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiNotLoadedCorrectlyTooltip',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
      }
      else if ( !schedulerInfo.isPublicApiCompatibleVersion() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiIncompatibleVersionTooltip',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            QDesk.util.FormatUtils.formatProductVersion( schedulerInfo.getPublicApiVersion() ),
            QDesk.util.FormatUtils.formatProductVersion( jvmAgentInfo.getVersion() ) );
      }
      else
      {
        return null;
      }
    }
    else
    {
      return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiNotDetectedTooltip',
          QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
    }
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      qdesk : {
        // tab initialization flag
        initialized : false
      }
    } );

    me.callParent( [ config ] );
  }
} );
