/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.jobs.misfire.JobMisfiredTriggersView', {
  extend : 'QDesk.view.work.scheduler.BaseSchedulerSubTab',

  requires : [
    'QDesk.JvmAgentInfoAccessor',
    'QDesk.Msg',
    'QDesk.util.license.LicensedProduct',
    'QDesk.util.license.LicensedProductFeatures',
    'QDesk.view.widget.message.WarningMessagePanel',
    'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid'
  ],

  alias : 'widget.QuartzJobMisfiredTriggersView',

  iconCls : 'quartzJobMisfiredTriggersTabIcon',

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    var items = [];

    items.push(
        {
          xtype : 'WarningMessagePanel',
          style : {
            margin : '5px'
          },
          hidden : true
        } );

    items.push(
        {
          xtype : 'QuartzMisfiredTriggersGrid',
          itemId : 'misfiredTriggersGrid',
          flex : 1,
          hidden : true,
          qdesk : {
            gridId : 'quartz.jobMisfiredTriggersGrid',
            maxFetchSize : QDesk.ApplicationConfig.get( 'quartz.jobMisfiredTriggersGrid.maxFetchSize' )
          }
        } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var warningVisible = false;
    var warningMsg = null;
    var gridVisible = false;

    var jvmAgentInfo = config.qdesk.jvmAgentInfo;

    if ( licensedFeatures.MisfiredTriggers )
    {
      if ( jvmAgentInfo.isReady() )
      {
        if ( jvmAgentInfo.isJvmAgentQuartzMisfiredTriggersTrackingEnabled() )
        {
          if ( me.isPublicApiReady( config.qdesk.schedulerInfo ) )
          {
            if ( jvmAgentInfo.isJvmAgentQuartzMisfiredTriggersProcessingEnabled() )
            {
              // everything is OK
              gridVisible = true;
            }
            else
            {
              // misfired trigger processing is not enabled
              warningVisible = true;
              warningMsg = QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.jobs.misfire.JobMisfiredTriggersView.misfiredTriggersProcessingDisabledMsg' );

              // grid visible because misfired triggers are tracked (inserted), they are just not processed
              gridVisible = true;
            }
          }
          else
          {
            // public API is not ready
            warningVisible = true;
            warningMsg = me.getPublicApiWarningMessage( config.qdesk.schedulerInfo, config.qdesk.jvmAgentInfo );
            gridVisible = true;
          }
        }
        else
        {
          // misfired trigger tracking is not enabled
          warningVisible = true;
          warningMsg = QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.misfire.JobMisfiredTriggersView.misfiredTriggersTrackingDisabledMsg' );

          // grid hidden because tracking of misfired triggers is disabled in the agent
          gridVisible = false;
        }
      }
      else
      {
        warningVisible = true;
        warningMsg = me.getJvmAgentWarningMessage( config.qdesk.schedulerInfo, config.qdesk.jvmAgentInfo );

        // grid hidden because agent is not ready
        gridVisible = false;
      }
    }
    else
    {
      warningVisible = true;
      warningMsg = QDesk.Msg.get( 'QDesk.featureAccessRestricted.notLicensedMsg',
          QDesk.util.license.LicensedProduct.productName )
    }

    /*
     * Intentionally over-nested a bit so that we use identical layout to other tabs that
     * have a license restrictions related message panel above the grid.
     */
    Ext.apply( config, {
      title : me._getTitle(),
      tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.misfire.JobMisfiredTriggersView.tooltip' ),
      items : [
        {
          xtype : 'panel',
          border : false,
          layout : 'fit',
          flex : 1,
          items : [
            {
              xtype : 'panel',
              border : false,
              layout : {
                type : 'vbox',
                align : 'stretch'
              },
              items : items
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );

    if ( warningVisible )
    {
      var warningPanel = me.down( 'WarningMessagePanel' );
      warningPanel.setMessage( warningMsg );
      warningPanel.setVisible( true );
    }

    if ( gridVisible )
    {
      me.down( '#misfiredTriggersGrid' ).setVisible( true );
    }
  },


  /**
   * Overridden method from the base class that returns the warning message shown at the top of the misfired triggers
   * grid.
   * @param schedulerInfo the scheduler info.
   * @param jvmAgentInfo the QuartzDesk JVM Agent info.
   * @returns the warning message.
   */
  getPublicApiWarningMessage : function ( schedulerInfo, jvmAgentInfo )
  {
    if ( schedulerInfo.isPublicApiReady() )
      return null;

    if ( schedulerInfo.isPublicApiDetected() )
    {
      if ( !schedulerInfo.isPublicApiLoadedCorrectly() )
      {
        return QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.jobs.misfire.JobMisfiredTriggersView.publicApiNotLoadedCorrectlyMsg' );
      }
      else if ( !schedulerInfo.isPublicApiCompatibleVersion() )
      {
        return QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.jobs.misfire.JobMisfiredTriggersView.publicApiIncompatibleVersionMsg',
            QDesk.util.FormatUtils.formatProductVersion( schedulerInfo.getPublicApiVersion() ),
            QDesk.util.FormatUtils.formatProductVersion( jvmAgentInfo.getVersion() ) );
      }
      else
      {
        return null;
      }
    }
    else
    {
      return QDesk.Msg.get(
          'QDesk.view.work.scheduler.quartz.jobs.misfire.JobMisfiredTriggersView.publicApiNotDetectedMsg' );
    }
  },


  _getTitle : function ()
  {
    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var title = QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.misfire.JobMisfiredTriggersView.title' );
    if ( !licensedFeatures.MisfiredTriggers )
    {
      title = '<span class="notLicensed">' + title + '</span>'
    }

    return title;
  }
} );